@@ -18,7 +18,6 @@
 # ======================================================================
 
 import re
-
 from typing import Any, Dict, List, Sequence
 
 from supervisor.loggers import Logger
@@ -56,7 +55,6 @@ class ApplicationRules(object):
         self.logger: Logger = supvisors.logger
         # attributes
         self.managed: bool = False
-        self.distributed: bool = True
         self.distribution: DistributionRules = DistributionRules.ALL_INSTANCES
         self.identifiers: NameList = ['*']
         self.hash_identifiers: NameList = []
@@ -102,7 +100,7 @@ class ApplicationRules(object):
             if application_number < 0:
                 self.logger.error(f'ApplicationRules.check_hash_identifiers: index of {application_name} must be > 0')
             else:
-                self.logger.debug(f'ApplicationRules.check_hash_identifiers: application_name={application_name}'
+                self.logger.trace(f'ApplicationRules.check_hash_identifiers: application_name={application_name}'
                                   f' application_number={application_number}')
                 if '*' in self.hash_identifiers:
                     # all identifiers defined in the supvisors section of the supervisor configuration are applicable
@@ -110,14 +108,14 @@ class ApplicationRules(object):
                 else:
                     # the subset of applicable identifiers is the hash_identifiers
                     ref_identifiers = self.hash_identifiers
-                if application_number < len(ref_identifiers):
-                    self.identifiers = [ref_identifiers[application_number]]
-                    error = False
-                else:
-                    self.logger.error(f'ApplicationRules.check_hash_identifiers: {application_name} has no'
-                                      ' applicable identifier')
+                # if there are more application instances than possible identifiers, roll over
+                index = application_number % len(ref_identifiers)
+                self.identifiers = [ref_identifiers[index]]
+                self.logger.debug(f'ApplicationRules.check_hash_identifiers: application_name={application_name}'
+                                  f' identifiers={self.identifiers}')
+                error = False
         if error:
-            self.logger.warn(f'ApplicationRules.check_hash_identifiers: {application_name} start_sequence reset')
+            self.logger.debug(f'ApplicationRules.check_hash_identifiers: {application_name} start_sequence reset')
             self.start_sequence = 0
 
     def check_dependencies(self, application_name: str) -> None:
@@ -149,8 +147,7 @@ class ApplicationRules(object):
         :return: the application rules in a dictionary
         """
         if self.managed:
-            payload = {'managed': True, 'distributed': self.distribution == DistributionRules.ALL_INSTANCES,  # DEPRECATED
-                       'distribution': self.distribution.name, 'identifiers': self.identifiers,  # TODO: add / replace by nodes / targets ?
+            payload = {'managed': True, 'distribution': self.distribution.name, 'identifiers': self.identifiers,
                        'start_sequence': self.start_sequence, 'stop_sequence': self.stop_sequence,
                        'starting_strategy': self.starting_strategy.name,
                        'starting_failure_strategy': self.starting_failure_strategy.name,
@@ -202,7 +199,7 @@ class ApplicationStatus(object):
         self.start_sequence: ApplicationStatus.ApplicationSequence = {}
         self.stop_sequence: ApplicationStatus.ApplicationSequence = {}
 
-    # access
+    # status methods
     def running(self) -> bool:
         """ Return True if the application is running.
 
@@ -284,7 +281,7 @@ class ApplicationStatus(object):
         return (f'application_name={self.application_name} managed={self.rules.managed} state={self.state.name}'
                 f' major_failure={self.major_failure} minor_failure={self.minor_failure}')
 
-    # methods
+    # process methods
     def add_process(self, process: ProcessStatus) -> None:
         """ Add a new process to the process list.
 
@@ -294,7 +291,7 @@ class ApplicationStatus(object):
         self.processes[process.process_name] = process
 
     def remove_process(self, process_name: str) -> None:
-        """ Add a new process to the process list.
+        """ Remove a process from the process list.
 
         :param process_name: the process to be removed from the application
         :return: None
@@ -307,22 +304,34 @@ class ApplicationStatus(object):
 
     def possible_identifiers(self) -> NameList:
         """ Return the list of Supervisor identifiers where the application could be started.
-        To achieve that, two conditions:
+        To achieve that, three conditions:
             - the Supervisor must know all the application programs ;
-            - the Supervisor identifier must be declared in the rules file.
+            - the Supervisor identifier must be declared in the rules file ;
+            - the programs shall not be disabled.
 
         :return: the list of identifiers where the program could be started
         """
         identifiers = self.rules.identifiers
         if '*' in self.rules.identifiers:
-            identifiers = self.supvisors.supvisors_mapper.instances
+            identifiers = list(self.supvisors.supvisors_mapper.instances.keys())
         # get the identifiers common to all application processes
-        actual_identifiers = [set(process.info_map.keys()) for process in self.processes.values()]
+        actual_identifiers = [{identifier for identifier, info in process.info_map.items() if not info['disabled']}
+                              for process in self.processes.values()]
         if actual_identifiers:
             actual_identifiers = actual_identifiers[0].intersection(*actual_identifiers)
         # intersect with rules
         return [identifier for identifier in identifiers if identifier in actual_identifiers]
 
+    def get_instance_processes(self, identifier: str) -> ProcessList:
+        """ Return the list of application processes configured in the Supervisor instance.
+
+        :param identifier: the Supervisor instance where the processes qre configured
+        :return: the list of application processes in the Supervisor instance
+        """
+        return [process for process in self.processes.values()
+                if identifier in process.info_map]
+
+    # sequencing methods
     @staticmethod
     def printable_sequence(application_sequence: ApplicationSequence) -> PrintableApplicationSequence:
         """ Get printable application sequence for log traces.
@@ -376,6 +385,7 @@ class ApplicationStatus(object):
         """
         return sum(process.rules.expected_load for process in self.get_start_sequenced_processes())
 
+    # global status methods
     def update_status(self) -> None:
         """ Update the state of the application iaw the state of its sequenced processes.
         An unmanaged application - or generally without starting sequence - is always STOPPED.
@@ -407,7 +417,7 @@ class ApplicationStatus(object):
                 stopping = True
             elif (process.state in [ProcessStates.FATAL, ProcessStates.UNKNOWN]
                     or process.state == ProcessStates.EXITED and not process.expected_exit):
-                # a major/minor failure is raised in this states depending on the required option
+                # a major/minor failure is raised in these states depending on the required option
                 if process.rules.required:
                     self.major_failure = True
                 else:
