@@ -20,26 +20,56 @@
 import sys
 
 from supervisor import supervisord
-from supervisor.loggers import getLogger, handle_file, handle_stdout
+from supervisor.loggers import Logger, getLogger, handle_file, handle_stdout
 from supervisor.supervisord import Supervisor
-from supervisor.xmlrpc import Faults, RPCError
 
-from .supvisorsmapper import SupvisorsMapper
 from .commander import Starter, Stopper
 from .context import Context
-from .supervisordata import SupervisorData
 from .listener import SupervisorListener
-from .options import *
+from .options import SupvisorsOptions, SupvisorsServerOptions, Automatic, get_logger_configuration
 from .sparser import Parser
 from .statemachine import FiniteStateMachine
 from .statscompiler import StatisticsCompiler
 from .strategy import RunningFailureHandler
+from .supervisordata import SupervisorData
+from .supvisorsmapper import SupvisorsMapper
+from .ttypes import Payload
+
+
+def create_logger(supervisor: Supervisor, logger_config: Payload) -> Logger:
+    """ Create the logger that will be used in Supvisors.
+    If logfile is not set or set to AUTO, Supvisors will use Supervisor logger.
+    Else Supvisors will log in the file defined in option.
+    """
+    logfile = logger_config['logfile']
+    if logfile is Automatic:
+        # use Supervisord logger but patch format anyway
+        logger = supervisor.options.logger
+        for handler in logger.handlers:
+            handler.setFormat(Supvisors.LOGGER_FORMAT)
+        return logger
+    # else create own Logger using Supervisor functions
+    nodaemon = supervisor.options.nodaemon
+    silent = supervisor.options.silent
+    logger = getLogger(logger_config['loglevel'])
+    # tag the logger so that it is properly closed when exiting
+    # when not tagged, Supervisor closes it
+    logger.SUPVISORS = True
+    if nodaemon and not silent:
+        handle_stdout(logger, Supvisors.LOGGER_FORMAT)
+    handle_file(logger,
+                logfile,
+                Supvisors.LOGGER_FORMAT,
+                rotating=not not logger_config['logfile_maxbytes'],
+                maxbytes=logger_config['logfile_maxbytes'],
+                backups=logger_config['logfile_backups'])
+    return logger
 
 
 class Supvisors(object):
     """ The Supvisors class used as a global structure passed to most Supvisors objects. """
 
-    # logger output (use ';' as separator as easier to cut)
+    # use ';' in logger output as separator as easier to cut
     LOGGER_FORMAT = '%(asctime)s;%(levelname)s;%(message)s\n'
 
     def __init__(self, supervisor: Supervisor, **config) -> None:
@@ -47,24 +77,27 @@ class Supvisors(object):
 
         :param supervisor: the Supervisor global structure
         """
-        # declare zmq context (will be created in listener)
-        self.zmq = None
+        # WARN: the Supvisors sockets cannot be created at this level
+        # Before running, Supervisor forks when daemonized and the sockets would be lost
+        self.sockets = None
+        self.external_publisher = None
+        # create logger using option from config
+        logger_config = get_logger_configuration(**config)
+        self.logger = create_logger(supervisor, logger_config)
         # get options from config
-        self.options = SupvisorsOptions(supervisor, **config)
-        # create logger
-        self.logger = self.create_logger(supervisor)
-        # re-realize configuration to get
+        self.options = SupvisorsOptions(supervisor, self.logger, **config)
+        # re-realize configuration to get process configuration not stored within Supervisor options
         self.server_options = SupvisorsServerOptions(self.logger)
         self.server_options.realize(sys.argv[1:], doc=supervisord.__doc__)
-        # configure supervisor info source
-        self.supervisor_data = SupervisorData(supervisor, self.logger)
+        # configure supervisor data wrapper
+        self.supervisor_data = SupervisorData(self, supervisor)
         # get declared Supvisors instances and check local identifier
         self.supvisors_mapper = SupvisorsMapper(self)
         try:
             self.supvisors_mapper.configure(self.options.supvisors_list, self.options.core_identifiers)
         except ValueError as exc:
-            self.logger.critical(f'Supvisors: {exc}')
-            raise RPCError(Faults.SUPVISORS_CONF_ERROR, str(exc))
+            self.logger.critical('Wrong Supvisors configuration (supvisors_list)')
+            raise
         # create context data
         self.context = Context(self)
         # create application starter and stopper
@@ -75,8 +108,6 @@ class Supvisors(object):
         # create the failure handler of crashing processes
         # WARN: must be created before the state machine
         self.failure_handler = RunningFailureHandler(self)
-        # create state machine
-        self.fsm = FiniteStateMachine(self)
         # check parsing
         try:
             self.parser = Parser(self)
@@ -85,30 +116,5 @@ class Supvisors(object):
             self.parser = None
         # create event subscriber
         self.listener = SupervisorListener(self)
-
-    def create_logger(self, supervisor: Supervisor) -> Logger:
-        """ Create the logger that will be used in Supvisors.
-        If logfile is not set or set to AUTO, Supvisors will use Supervisor logger.
-        Else Supvisors will log in the file defined in option.
-        """
-        if self.options.logfile is Automatic:
-            # use Supervisord logger but patch format anyway
-            logger = supervisor.options.logger
-            for handler in logger.handlers:
-                handler.setFormat(Supvisors.LOGGER_FORMAT)
-            return logger
-        # else create own Logger using Supervisor functions
-        nodaemon = supervisor.options.nodaemon
-        silent = supervisor.options.silent
-        logger = getLogger(self.options.loglevel)
-        # tag the logger so that it is properly closed when exiting
-        logger.SUPVISORS = True
-        if nodaemon and not silent:
-            handle_stdout(logger, Supvisors.LOGGER_FORMAT)
-        handle_file(logger,
-                    self.options.logfile,
-                    Supvisors.LOGGER_FORMAT,
-                    rotating=not not self.options.logfile_maxbytes,
-                    maxbytes=self.options.logfile_maxbytes,
-                    backups=self.options.logfile_backups)
-        return logger
+        # create state machine
+        self.fsm = FiniteStateMachine(self)
