@@ -20,32 +20,51 @@
 import glob
 import os
 import platform
-
 from collections import OrderedDict
 from socket import gethostname
 from typing import Dict, List, TypeVar
 
-from supervisor.datatypes import (Automatic, logfile_name, boolean, integer, byte_size,
+from supervisor.datatypes import (Automatic, logfile_name, boolean, integer, byte_size, existing_dirpath,
                                   logging_level, list_of_strings)
 from supervisor.loggers import Logger
 from supervisor.options import expand, ServerOptions, ProcessConfig, FastCGIProcessConfig, EventListenerConfig
 
-from .ttypes import ConciliationStrategies, StartingStrategies, enum_names
+from .ttypes import ConciliationStrategies, EventLinks, Payload, StartingStrategies
 
 
 # Options of main section
-class SupvisorsOptions(object):
+def get_logger_configuration(**config) -> Payload:
+    """ Extract the logger parameters from the config structure.
+
+    Returns a dictionary with the following entries:
+        - logfile: absolute or relative path of the Supvisors log file ;
+        - logfile_maxbytes: maximum size of the Supvisors log file ;
+        - logfile_backups: number of Supvisors backup log files ;
+        - loglevel: logging level.
+
+    :param config: the configuration provided by Supervisor from the [rpcinterface:supvisors] section
+    :return: a dictionary containing the logger parameters
+    """
+    return {'logfile': logfile_name(config.get('logfile', Automatic)),
+            'logfile_maxbytes': byte_size(config.get('logfile_maxbytes', '50MB')),
+            'logfile_backups': integer(config.get('logfile_backups', 10)),
+            'loglevel': logging_level(config.get('loglevel', 'info'))}
+
+
+class SupvisorsOptions:
     """ Holder of the Supvisors options.
 
     Attributes are:
         - supvisors_list: list of Supvisors instance identifiers where Supvisors will be running ;
         - rules_files: list of absolute or relative paths to the XML rules files ;
         - internal_port: port number used to publish local events to remote Supvisors instances ;
+        - event_link: type of the event link used to publish all Supvisors events ;
         - event_port: port number used to publish all Supvisors events ;
         - auto_fence: when True, Supvisors won't try to reconnect to a Supvisors instance that has been inactive ;
         - synchro_timeout: time in seconds that Supvisors waits for all expected Supvisors instances to publish ;
         - inactivity_ticks: number of local ticks to wait before considering a remote Supvisors instance inactive ;
         - core_identifiers: subset of supvisors_list identifiers that will force the end of synchro when all RUNNING ;
+        - disabilities_file: the file used to persist the process disabilities ;
         - conciliation_strategy: strategy used to solve conflicts when Supvisors has detected multiple running
           instances of the same program ;
         - starting_strategy: strategy used to start processes on Supvisors instances ;
@@ -53,10 +72,8 @@ class SupvisorsOptions(object):
         - stats_periods: list of periods for which the statistics will be provided in the Supvisors Web UI ;
         - stats_histo: depth of statistics history ;
         - stats_irix_mode: choice of CPU value display between IRIX and Solaris ;
-        - logfile: absolute or relative path of the Supvisors log file ;
-        - logfile_maxbytes: maximum size of the Supvisors log file ;
-        - logfile_backups: number of Supvisors backup log files ;
-        - loglevel: logging level.
+        - tail_limit: the number of bytes used to display the log tail of the file in the Web UI (refresh mode) ;
+        - tailf_limit: the number of bytes used to display the log tail of the file in the Web UI (tail -f mode).
     """
 
     SYNCHRO_TIMEOUT_MIN = 15
@@ -65,56 +82,78 @@ class SupvisorsOptions(object):
     INACTIVITY_TICKS_MIN = 2
     INACTIVITY_TICKS_MAX = 720
 
-    def __init__(self, supervisord, **config):
+    def __init__(self, supervisord, logger: Logger, **config):
         """ Initialization of the attributes.
 
         :param supervisord: the global Supervisor structure
+        :param logger: the Supvisors logger
         :param config: the configuration provided by Supervisor from the [rpcinterface:supvisors] section
         """
         self.supervisord_options = supervisord.options
+        self.logger = logger
         # get values from config
-        supvisors_list = config.get('supvisors_list', gethostname())
-        supvisors_list = filter(None, list_of_strings(supvisors_list))
-        self.supvisors_list = list(OrderedDict.fromkeys(supvisors_list))
-        # keep rules_file for next version but state obsolescence
-        self.rules_files = config.get('rules_files', None)
-        if self.rules_files:
-            self.rules_files = self.to_filepaths(self.rules_files)
+        self.supvisors_list = self._get_value(config, 'supvisors_list', [gethostname()],
+                                              lambda x: list(OrderedDict.fromkeys(filter(None, list_of_strings(x)))))
+        self.rules_files = self._get_value(config, 'rules_files', None, self.to_filepaths)
         # if internal_port and event_port are not defined, they will be set later based on Supervisor HTTP port
-        self.internal_port = self.to_port_num(config.get('internal_port', '0'))
-        self.event_port = self.to_port_num(config.get('event_port', '0'))
-        self.auto_fence = boolean(config.get('auto_fence', 'false'))
-        self.synchro_timeout = self.to_timeout(config.get('synchro_timeout', str(self.SYNCHRO_TIMEOUT_MIN)))
-        self.inactivity_ticks = self.to_ticks(config.get('inactivity_ticks', str(self.INACTIVITY_TICKS_MIN)))
+        self.internal_port = self._get_value(config, 'internal_port', 0, self.to_port_num)
+        self.event_link = self._get_value(config, 'event_link', EventLinks.NONE, self.to_event_link)
+        self.event_port = self._get_value(config, 'event_port', 0, self.to_port_num)
+        self.auto_fence = self._get_value(config, 'auto_fence', False, boolean)
+        self.synchro_timeout = self._get_value(config, 'synchro_timeout', self.SYNCHRO_TIMEOUT_MIN, self.to_timeout)
+        self.inactivity_ticks = self._get_value(config, 'inactivity_ticks', self.INACTIVITY_TICKS_MIN, self.to_ticks)
         # get the minimum list of identifiers to end the synchronization phase
-        core_identifiers = config.get('core_identifiers', None)
-        self.core_identifiers = set(filter(None, list_of_strings(core_identifiers)))
+        self.core_identifiers = self._get_value(config, 'core_identifiers', set(),
+                                                lambda x: set(filter(None, list_of_strings(x))))
+        # get disabilities file
+        self.disabilities_file = self._get_value(config, 'disabilities_file', None, existing_dirpath)
         # get strategies
-        self.conciliation_strategy = self.to_conciliation_strategy(config.get('conciliation_strategy', 'USER'))
-        self.starting_strategy = self.to_starting_strategy(config.get('starting_strategy', 'CONFIG'))
+        self.conciliation_strategy = self._get_value(config, 'conciliation_strategy', ConciliationStrategies.USER,
+                                                     self.to_conciliation_strategy)
+        self.starting_strategy = self._get_value(config, 'starting_strategy', StartingStrategies.CONFIG,
+                                                 self.to_starting_strategy)
         # configure statistics
-        self.stats_enabled = boolean(config.get('stats_enabled', 'true'))
-        self.stats_periods = self.to_periods(list_of_strings(config.get('stats_periods', '10')))
-        self.stats_histo = self.to_histo(config.get('stats_histo', 200))
-        self.stats_irix_mode = boolean(config.get('stats_irix_mode', 'false'))
-        # configure logger
-        self.logfile = logfile_name(config.get('logfile', Automatic))
-        self.logfile_maxbytes = byte_size(config.get('logfile_maxbytes', '50MB'))
-        self.logfile_backups = integer(config.get('logfile_backups', 10))
-        self.loglevel = logging_level(config.get('loglevel', 'info'))
+        self.stats_enabled = self._get_value(config, 'stats_enabled', True, boolean)
+        self.stats_periods = self._get_value(config, 'stats_periods', [10], self.to_periods)
+        self.stats_histo = self._get_value(config, 'stats_histo', 200, self.to_histo)
+        self.stats_irix_mode = self._get_value(config, 'stats_irix_mode', False, boolean)
+        # configure log tail limits
+        self.tail_limit = self._get_value(config, 'tail_limit', 1024, byte_size)
+        self.tailf_limit = self._get_value(config, 'tailf_limit', 1024, byte_size)
 
     def __str__(self):
         """ Contents as string. """
         return (f'supvisors_list={self.supvisors_list} rules_files={self.rules_files}'
-                f' internal_port={self.internal_port} event_port={self.event_port} auto_fence={self.auto_fence}'
+                f' internal_port={self.internal_port}'
+                f' event_link={self.event_link.name} event_port={self.event_port}'
+                f' auto_fence={self.auto_fence}'
                 f' synchro_timeout={self.synchro_timeout} inactivity_ticks={self.inactivity_ticks}'
-                f' core_identifiers={self.core_identifiers}'
+                f' core_identifiers={self.core_identifiers} disabilities_file={self.disabilities_file}'
                 f' conciliation_strategy={self.conciliation_strategy.name}'
                 f' starting_strategy={self.starting_strategy.name}'
                 f' stats_enabled={self.stats_enabled} stats_periods={self.stats_periods} stats_histo={self.stats_histo}'
                 f' stats_irix_mode={self.stats_irix_mode}'
-                f' logfile={self.logfile} logfile_maxbytes={self.logfile_maxbytes}'
-                f' logfile_backups={self.logfile_backups} loglevel={self.loglevel}')
+                f' tail_limit={self.tail_limit} tailf_limit={self.tailf_limit}')
+
+    def _get_value(self, config: Payload, attr: str, default_value, fct=None):
+        """ Read and convert the option.
+
+        :param config: the option dictionary
+        :param attr: the option considered
+        :param default_value: the default value to apply if not found in config or erroneous
+        :param fct: the optional conversion function to apply to the string value
+        :return:
+        """
+        if attr not in config:
+            return default_value
+        value = config[attr]
+        if fct:
+            try:
+                return fct(value)
+            except ValueError as exc:
+                self.logger.error(f'SupvisorsOptions.get_value: {str(exc)}')
+                return default_value
+        return value
 
     # conversion utils (completion of supervisor.datatypes)
     def to_filepaths(self, value: str) -> List[str]:
@@ -134,6 +173,9 @@ class SupvisorsOptions(object):
             filepaths = glob.glob(pattern)
             for filepath in filepaths:
                 rules_files.add(os.path.abspath(filepath))
+        # check that something came out
+        if value and not rules_files:
+            self.logger.warn('SupvisorsOptions.to_filepaths: no rules file found')
         return sorted(rules_files)
 
     @staticmethod
@@ -146,7 +188,7 @@ class SupvisorsOptions(object):
         value = integer(value)
         if 0 <= value <= 65535:
             return value
-        raise ValueError(f'invalid value for port: {value}. expected in [0;65535]')
+        raise ValueError(f'invalid value for port ({value}), expected in [0;65535]')
 
     @staticmethod
     def to_timeout(value: str) -> int:
@@ -177,34 +219,45 @@ class SupvisorsOptions(object):
                          f'{SupvisorsOptions.INACTIVITY_TICKS_MAX}]')
 
     @staticmethod
-    def to_conciliation_strategy(value):
+    def to_event_link(value: str) -> EventLinks:
+        """ Convert a string into a EventLinks enum. """
+        try:
+            event_link = EventLinks[value.upper()]
+        except KeyError:
+            raise ValueError(f'invalid value for event_link: {value}.'
+                             f' expected in {[x.name for x in EventLinks]}')
+        return event_link
+
+    @staticmethod
+    def to_conciliation_strategy(value: str) -> ConciliationStrategies:
         """ Convert a string into a ConciliationStrategies enum. """
         try:
             strategy = ConciliationStrategies[value]
         except KeyError:
             raise ValueError(f'invalid value for conciliation_strategy: {value}.'
-                             f' expected in {enum_names(ConciliationStrategies)}')
+                             f' expected in {[x.name for x in ConciliationStrategies]}')
         return strategy
 
     @staticmethod
-    def to_starting_strategy(value):
+    def to_starting_strategy(value: str) -> StartingStrategies:
         """ Convert a string into a StartingStrategies enum. """
         try:
             strategy = StartingStrategies[value]
         except KeyError:
             raise ValueError(f'invalid value for starting_strategy: {value}.'
-                             f' expected in {enum_names(StartingStrategies)}')
+                             f' expected in {[x.name for x in StartingStrategies]}')
         return strategy
 
     @staticmethod
-    def to_periods(value):
+    def to_periods(value: str) -> List[int]:
         """ Convert a string into a list of period values. """
-        if len(value) == 0:
-            raise ValueError(f'unexpected number of stats_periods: {len(value)}. minimum is 1')
-        if len(value) > 3:
-            raise ValueError(f'unexpected number of stats_periods: {len(value)}. maximum is 3')
+        str_periods = list_of_strings(value)
+        if len(str_periods) == 0:
+            raise ValueError(f'unexpected number of stats_periods: {len(str_periods)}. minimum is 1')
+        if len(str_periods) > 3:
+            raise ValueError(f'unexpected number of stats_periods: {len(str_periods)}. maximum is 3')
         periods = []
-        for val in value:
+        for val in str_periods:
             try:
                 period = integer(val)
             except ValueError:
@@ -231,7 +284,16 @@ class SupvisorsOptions(object):
 
 
 class SupvisorsServerOptions(ServerOptions):
-    """ Class used to parse the options of the 'supvisors' section in the supervisor configuration file. """
+    """ Class used to parse the options of the 'supvisors' section in the supervisor configuration file.
+
+    Attributes are:
+        - parser: the config parser ;
+        - program_class: the Supervisor class type of the program among {ProcessConfig, FastCGIProcessConfig,
+          EventListenerConfig} ;
+        - program_processes: for each program, the group names using it and the corresponding process configurations ;
+        - process_programs: the program associated to each process (key is a process name, not a namespec).
+        - procnumbers: the index of each process (key is a process name, not a namespec).
+    """
 
     # annotation types
     ProcessConfigList = List[ProcessConfig]
@@ -248,7 +310,8 @@ class SupvisorsServerOptions(ServerOptions):
         # attributes
         self.parser = None
         self.program_class: SupvisorsServerOptions.ProcessClassInfo = {}
-        self.process_groups: SupvisorsServerOptions.ProcessGroupInfo = {}
+        self.program_processes: SupvisorsServerOptions.ProcessGroupInfo = {}
+        self.processes_program: Dict[str, str] = {}
         self.procnumbers: Dict[str, int] = {}
 
     def _processes_from_section(self, parser, section, group_name, klass=None) -> List[ProcessConfig]:
@@ -267,15 +330,16 @@ class SupvisorsServerOptions(ServerOptions):
         self.parser = parser
         # call super behaviour
         process_configs = ServerOptions._processes_from_section(self, parser, section, group_name, klass)
-        # store the number of each program
-        for idx, program in enumerate(process_configs):
-            self.procnumbers[program.name] = idx
         # store process configurations and groups
         program_name = section.split(':', 1)[1]
-        process_group = self.process_groups.setdefault(program_name, {})
-        process_group[group_name] = process_configs
+        program_groups = self.program_processes.setdefault(program_name, {})
+        program_groups[group_name] = process_configs
         # store the program class type
         self.program_class[program_name] = klass
+        # store the number and the program of each process
+        for idx, process_config in enumerate(process_configs):
+            self.processes_program[process_config.name] = program_name
+            self.procnumbers[process_config.name] = idx
         # return original result
         return process_configs
 
@@ -311,10 +375,11 @@ class SupvisorsServerOptions(ServerOptions):
         :param group_name: the group that embeds the program definition
         :return: the list of ProcessConfig
         """
-        # reset corresponding store procnumbers
+        # reset corresponding stored procnumbers
         program_name = section.split(':')[1]
-        for process_list in self.process_groups[program_name].values():
+        for process_list in self.program_processes[program_name].values():
             for process in process_list:
+                self.processes_program.pop(process.name, None)
                 self.procnumbers.pop(process.name, None)
         # call parser again
         klass = self.program_class[program_name]
