@@ -17,14 +17,14 @@
 # limitations under the License.
 # ======================================================================
 
-from typing import AbstractSet, Any, Dict, Optional, Set, Tuple
+from typing import AbstractSet, Any, Dict, Optional, Set
 
 from supervisor.loggers import Logger, LevelsByName
 from supervisor.options import make_namespec, split_namespec
 from supervisor.rpcinterface import SupervisorNamespaceRPCInterface
 from supervisor.states import ProcessStates, getProcessStateDescription, RUNNING_STATES, STOPPED_STATES
 
-from .ttypes import NameList, Payload, RunningFailureStrategies, StartingFailureStrategies
+from .ttypes import NameList, RunningFailureStrategies, StartingFailureStrategies
 from .utils import *
 
 
@@ -117,29 +117,26 @@ class ProcessRules(object):
         :param namespec: the namespec of the program considered.
         :return: None
         """
-        error = True
         _, process_name = split_namespec(namespec)
         try:
             procnumber = self.supvisors.server_options.procnumbers[process_name]
         except KeyError:
             self.logger.error(f'ProcessRules.check_hash_identifiers: cannot apply "#" to unknown program={namespec}')
+            self.logger.debug(f'ProcessRules.check_hash_identifiers: namespec={namespec} reset start_sequence')
+            self.start_sequence = 0
         else:
-            self.logger.debug(f'ProcessRules.check_hash_identifiers: namespec={namespec} procnumber={procnumber}')
+            self.logger.trace(f'ProcessRules.check_hash_identifiers: namespec={namespec} procnumber={procnumber}')
             if '*' in self.hash_identifiers:
                 # all identifiers defined in the supvisors section of the supervisor configuration file are applicable
                 ref_identifiers = list(self.supvisors.supvisors_mapper.instances.keys())
             else:
                 # the subset of applicable identifiers is the second element of rule 'identifiers'
                 ref_identifiers = self.hash_identifiers
-            if procnumber < len(ref_identifiers):
-                self.identifiers = [ref_identifiers[procnumber]]
-                error = False
-            else:
-                self.logger.error(f'ProcessRules.check_hash_identifiers: namespec={namespec} has no applicable'
-                                  ' Supvisors identifier')
-        if error:
-            self.logger.warn(f'ProcessRules.check_hash_identifiers: namespec={namespec} start_sequence reset')
-            self.start_sequence = 0
+            # if there are more program instances than possible identifiers, roll over
+            index = procnumber % len(ref_identifiers)
+            self.identifiers = [ref_identifiers[index]]
+            self.logger.debug(f'ProcessRules.check_hash_identifiers: namespec={namespec}'
+                              f' identifiers={self.identifiers}')
 
     def check_dependencies(self, namespec: str) -> None:
         """ Update rules after they have been read from the rules file.
@@ -214,7 +211,7 @@ class ProcessStatus(object):
         self.forced_state: Optional[ProcessStates] = None
         self.forced_reason: str = ''
         self.expected_exit: bool = True
-        self.last_event_time: int = 0
+        self.last_event_time: float = 0.0
         self._extra_args: str = ''
         # rules part
         self.rules: ProcessRules = rules
@@ -242,27 +239,32 @@ class ProcessStatus(object):
             self._state = new_state
             self.logger.info(f'ProcessStatus.state: {self.namespec} is {self.state_string()}')
 
-    def force_state(self, event: Payload) -> None:
+    def force_state(self, event: Payload) -> bool:
         """ Force the process state due to an unexpected event.
         This may be caused by a process command that has not been acknowledged in due time, so here is a final check
-        for the expected state, in the event where messages have crossed.
+        in the event where messages have crossed.
 
         :param event: the forced event
-        :return: None
+        :return: True if the process state has been forced
         """
-        reached = False
+        force_state = True
         # check current state on targeted identifier
+        # Note: identifier may correspond to the Master identifier when it did not find any candidate Supvisors instance
+        # to start the process and this process may not be configured in the Supervisor of the Master
         identifier = event['identifier']
         if identifier in self.info_map:
             instance_info = self.info_map[identifier]
-            reached = instance_info['state'] == event['state']
-        # if expected event not reached, apply forced state
-        if not reached:
-            self.last_event_time = int(time())
-            self.forced_state = event['forced_state']
+            force_state = instance_info['event_time'] <= event['now']
+        # apply forced state only if no event has been received since the forced state has been evaluated
+        if force_state:
+            self.last_event_time = time()
+            self.forced_state = event['state']
             self.forced_reason = event['spawnerr']
             self.logger.info(f'ProcessStatus.force_state: {self.namespec} is {self.state_string()}'
                              f' ({self.forced_reason})')
+        else:
+            self.logger.debug(f'ProcessStatus.force_state: forced event dismissed for {self.namespec}')
+        return force_state
 
     def reset_forced_state(self, state: ProcessStates = None):
         """ Reset forced_state upon reception of new information only if not STOPPED (default state in Supervisor).
@@ -311,21 +313,39 @@ class ProcessStatus(object):
                 'extra_args': self.extra_args}
 
     # access
+    def disabled(self) -> bool:
+        """ Check if the process is disabled on all Supvisors instances knowing the process.
+
+        :return: the disabled status of the process
+        """
+        return self.info_map and all(info['disabled'] for info in self.info_map.values())
+
+    def disabled_on(self, identifier: str) -> bool:
+        """ Check if the process is disabled on the Supvisors instance identified.
+        If there is no information about the Supvisors instance, it is not considered as disabled.
+
+        :param identifier: the Supvisors instance identifier
+        :return: the disabled status of the process on the considered Supvisors instance
+        """
+        return identifier in self.info_map and self.info_map[identifier]['disabled']
+
     def possible_identifiers(self) -> NameList:
         """ Return the list of identifier where the program could be started.
-        To achieve that, two conditions:
+        To achieve that, three conditions:
             - the Supervisor of the Supvisors instance must know the program ;
-            - the Supvisors identifier must be declared in the rules file.
+            - the Supvisors identifier must be declared in the rules file ;
+            - the program shall not be disabled.
 
         :return: the list of identifiers where the program could be started
         """
         identifiers = self.rules.identifiers
         if '*' in self.rules.identifiers:
-            identifiers = self.supvisors.supvisors_mapper.instances
-        return [identifier for identifier in identifiers if identifier in self.info_map]
+            identifiers = list(self.supvisors.supvisors_mapper.instances.keys())
+        return [identifier for identifier in identifiers
+                if identifier in self.info_map and not self.disabled_on(identifier)]
 
     def has_crashed(self) -> bool:
-        """ Return True if the any of the processes has ever crashed or has ever exited unexpectedly.
+        """ Return True if any of the processes has ever crashed or has ever exited unexpectedly.
 
         :return: the crash status of the process
         """
@@ -446,8 +466,9 @@ class ProcessStatus(object):
         info = self.info_map[identifier] = payload
         # keep date of last information received
         # use local time here as there is no guarantee that Supvisors instances will be time-synchronized
-        self.last_event_time = int(time())
+        self.last_event_time = time()
         info['local_time'] = self.last_event_time
+        info['event_time'] = info['now']
         self.update_uptime(info)
         # WARN: when a new Supvisors instance comes in group, extra_args is kept only if information ties in
         if self.extra_args != info['extra_args']:
@@ -481,8 +502,9 @@ class ProcessStatus(object):
         info.update(payload)
         # keep date of last information received
         # use local time here as there is no guarantee that Supervisors will be time synchronized
-        self.last_event_time = int(time())
+        self.last_event_time = time()
         info['local_time'] = self.last_event_time
+        info['event_time'] = info['now']
         # last received extra_args are always applicable
         self.extra_args = payload['extra_args']
         # complete missing information
@@ -507,6 +529,16 @@ class ProcessStatus(object):
         # update / check running Supervisors
         self.update_status(identifier, new_state)
 
+    def update_disability(self, identifier: str, disabled: bool) -> None:
+        """ Update the disabled status of the process.
+
+        :param identifier: the identifier of the Supvisors instance from which the disability has been received
+        :param disabled: set to True if the process is disabled
+        :return: None
+        """
+        if identifier in self.info_map:
+            self.info_map[identifier]['disabled'] = disabled
+
     def update_times(self, identifier: str, remote_time: float) -> None:
         """ Update the internal process information when a new tick is received from the remote Supvisors instance.
 
