@@ -18,7 +18,6 @@
 # ======================================================================
 
 import re
-
 from distutils.util import strtobool
 from os import path
 from sys import stderr
@@ -30,7 +29,7 @@ from supervisor.options import split_namespec
 from .application import ApplicationRules
 from .process import ProcessRules
 from .ttypes import (DistributionRules, StartingStrategies, StartingFailureStrategies,
-                     RunningFailureStrategies, EnumClassType, enum_names)
+                     RunningFailureStrategies, EnumClassType)
 
 # XSD for XML validation
 supvisors_folder = path.dirname(__file__)
@@ -69,7 +68,7 @@ class Parser(object):
         :return: None
         """
         for rules_file in rules_files:
-            self.logger.info('Parser: parsing rules from {}'.format(rules_file))
+            self.logger.info(f'Parser: parsing rules from {rules_file}')
             root = self.parse(rules_file).getroot()
             self.roots.append(root)
             # get aliases - keep string as it is easier to process
@@ -83,16 +82,7 @@ class Parser(object):
             app_elements = root.findall('./application[@pattern]')
             self.application_patterns.update({app_element.get('pattern'): app_element
                                               for app_element in app_elements})
-            # DEPRECATED: get program patterns sorted by application
-            app_elements = root.findall('./application/program[@pattern]/..')
-            for app_element in app_elements:
-                prg_elements = app_element.findall('./program[@pattern]')
-                prg_patterns = self.program_patterns.setdefault(app_element, {})
-                prg_patterns.update({prg_element.get('pattern'): prg_element
-                                     for prg_element in prg_elements})
-                self.logger.warn(f'Parser.load_rules_files: DEPRECATED move program definitions'
-                                 ' into a <programs> section')
-            # NEW: get program patterns sorted by application
+            # get program patterns sorted by application
             app_elements = root.findall('./application/programs/program[@pattern]/../..')
             for app_element in app_elements:
                 prg_elements = app_element.findall('./programs/program[@pattern]')
@@ -125,12 +115,6 @@ class Parser(object):
         application_elt = self.get_application_element(application_name)
         if application_elt is not None:
             rules.managed = True
-            # TODO: DEPRECATED
-            if application_elt.findtext('distributed'):
-                self.logger.warn(f'Parser.load_application_rules: DEPRECATED element distributed in {application_name}')
-                self.load_boolean(application_elt, 'distributed', rules)
-                rules.distribution = (DistributionRules.ALL_INSTANCES if rules.distributed
-                                      else DistributionRules.SINGLE_INSTANCE)
             self.load_enum(application_elt, 'distribution', DistributionRules, rules)
             self.load_identifiers(application_elt, rules)
             self.load_sequence(application_elt, 'start_sequence', rules)
@@ -241,13 +225,6 @@ class Parser(object):
         program_elt = application_elt.find(f'./programs/program[@name="{process_name}"]')
         self.logger.trace(f'Parser.get_program_element: direct search for program={namespec} found'
                           f' {program_elt is not None}')
-        if program_elt is None:
-            program_elt = application_elt.find(f'./program[@name="{process_name}"]')
-            self.logger.trace(f'Parser.get_program_element: direct search for program={namespec} found'
-                              f' {program_elt is not None}')
-            if program_elt is not None:
-                self.logger.warn(f'Parser.get_program_element: DEPRECATED move program definitions'
-                                 ' into a <programs> section')
         if program_elt is None:
             # if not found as it is, try to find a corresponding pattern
             if application_elt in self.program_patterns:
@@ -417,7 +394,7 @@ class Parser(object):
                 setattr(rules, attr_string, klass[value])
             except KeyError:
                 self.logger.warn(f'Pattern.load_enum: invalid value for {Parser.get_element_name(elt)}'
-                                 f' {attr_string}: {value} (expected in {enum_names(klass)})')
+                                 f' {attr_string}: {value} (expected in {[x.name for x in klass]})')
 
     def parse(self, filename: str) -> Optional[Any]:
         """ Parse the file depending on the modules installed.
