@@ -17,13 +17,12 @@
 # limitations under the License.
 # ======================================================================
 
-import pytest
 import random
-
 from unittest.mock import call
 
-from supvisors.application import *
+import pytest
 
+from supvisors.application import *
 from .base import database_copy, any_process_info, any_stopped_process_info, any_process_info_by_state
 from .conftest import create_application, create_process
 
@@ -39,7 +38,7 @@ def test_rules_create(rules):
     """ Test the values set at construction. """
     # check application default rules
     assert not rules.managed
-    assert rules.distributed
+    assert rules.distribution == DistributionRules.ALL_INSTANCES
     assert rules.identifiers == ['*']
     assert rules.hash_identifiers == []
     assert rules.start_sequence == 0
@@ -101,8 +100,8 @@ def test_rules_check_hash_identifiers(rules):
     rules.hash_identifiers = ['10.0.0.1']
     rules.identifiers = []
     rules.check_hash_identifiers('sample_test_2')
-    assert rules.identifiers == []
-    assert rules.start_sequence == 0
+    assert rules.identifiers == ['10.0.0.1']
+    assert rules.start_sequence == 1
 
 
 def test_rules_check_dependencies(mocker, rules):
@@ -134,15 +133,13 @@ def test_rules_serial(rules):
     assert rules.serial() == {'managed': False}
     # check managed and distributed
     rules.managed = True
-    assert rules.serial() == {'managed': True, 'distributed': True,  # TODO: DEPRECATED
-                              'distribution': 'ALL_INSTANCES', 'identifiers': ['*'],
+    assert rules.serial() == {'managed': True, 'distribution': 'ALL_INSTANCES', 'identifiers': ['*'],
                               'start_sequence': 0, 'stop_sequence': -1,
                               'starting_strategy': 'CONFIG', 'starting_failure_strategy': 'ABORT',
                               'running_failure_strategy': 'CONTINUE'}
     # finally check managed and not distributed
     rules.distribution = DistributionRules.SINGLE_INSTANCE
-    assert rules.serial() == {'managed': True, 'distributed': False,  # TODO: DEPRECATED
-                              'distribution': 'SINGLE_INSTANCE', 'identifiers': ['*'],
+    assert rules.serial() == {'managed': True, 'distribution': 'SINGLE_INSTANCE', 'identifiers': ['*'],
                               'start_sequence': 0, 'stop_sequence': -1,
                               'starting_strategy': 'CONFIG', 'starting_failure_strategy': 'ABORT',
                               'running_failure_strategy': 'CONTINUE'}
@@ -162,7 +159,7 @@ def test_application_create(supvisors):
     assert not application.stop_sequence
     # check application default rules
     assert not application.rules.managed
-    assert application.rules.distributed
+    assert application.rules.distribution == DistributionRules.ALL_INSTANCES
     assert application.rules.start_sequence == 0
     assert application.rules.stop_sequence == -1
     assert application.rules.starting_failure_strategy == StartingFailureStrategies.ABORT
@@ -303,13 +300,13 @@ def test_application_possible_identifiers(supvisors):
     info = any_process_info_by_state(ProcessStates.STARTING)
     process1 = create_process(info, supvisors)
     for node_name in ['10.0.0.2', '10.0.0.3', '10.0.0.4']:
-        process1.add_info(node_name, info)
+        process1.add_info(node_name, info.copy())
     application.add_process(process1)
     # add another process to the application
     info = any_stopped_process_info()
     process2 = create_process(info, supvisors)
     for node_name in ['10.0.0.1', '10.0.0.4']:
-        process2.add_info(node_name, info)
+        process2.add_info(node_name, info.copy())
     application.add_process(process2)
     # default identifiers is '*' in process rules
     assert application.possible_identifiers() == ['10.0.0.4']
@@ -317,21 +314,48 @@ def test_application_possible_identifiers(supvisors):
     application.rules.identifiers = ['10.0.0.1', '10.0.0.2']
     assert application.possible_identifiers() == []
     # increase received status
-    process1.add_info('10.0.0.1', info)
+    process1.add_info('10.0.0.1', info.copy())
     assert application.possible_identifiers() == ['10.0.0.1']
+    # disable program on '10.0.0.1'
+    process2.update_disability('10.0.0.1', True)
+    assert application.possible_identifiers() == []
     # reset rules
     application.rules.identifiers = ['*']
-    assert application.possible_identifiers() == ['10.0.0.1', '10.0.0.4']
-    # test with full status and all instances in rules
+    assert application.possible_identifiers() == ['10.0.0.4']
+    # test with full status and all instances in rules + re-enable on '10.0.0.1'
+    process2.update_disability('10.0.0.1', False)
     for node_name in supvisors.supvisors_mapper.instances:
-        process1.add_info(node_name, info)
-        process2.add_info(node_name, info)
+        process1.add_info(node_name, info.copy())
+        process2.add_info(node_name, info.copy())
     assert sorted(application.possible_identifiers()) == sorted(supvisors.supvisors_mapper.instances.keys())
     # restrict again instances in rules
     application.rules.identifiers = ['10.0.0.5']
     assert application.possible_identifiers() == ['10.0.0.5']
 
 
+def test_application_get_instance_processes(supvisors):
+    """ Test the ApplicationStatus.get_instance_processes method. """
+    application = create_application('ApplicationTest', supvisors)
+    # add a process to the application
+    info = any_process_info_by_state(ProcessStates.STARTING)
+    process1 = create_process(info, supvisors)
+    for node_name in ['10.0.0.2', '10.0.0.3', '10.0.0.4']:
+        process1.add_info(node_name, info.copy())
+    application.add_process(process1)
+    # add another process to the application
+    info = any_stopped_process_info()
+    process2 = create_process(info, supvisors)
+    for node_name in ['10.0.0.1', '10.0.0.4']:
+        process2.add_info(node_name, info.copy())
+    application.add_process(process2)
+    # test call
+    assert application.get_instance_processes('10.0.0.0') == []
+    assert application.get_instance_processes('10.0.0.1') == [process2]
+    assert application.get_instance_processes('10.0.0.2') == [process1]
+    assert application.get_instance_processes('10.0.0.3') == [process1]
+    assert application.get_instance_processes('10.0.0.4') == [process1, process2]
+
+
 @pytest.fixture
 def filled_application(supvisors):
     """ Create an ApplicationStatus and add all processes of the database. """
