@@ -17,12 +17,11 @@
 # limitations under the License.
 # ======================================================================
 
-import pytest
-
 from unittest.mock import Mock
 
-from supvisors.initializer import *
+import pytest
 
+from supvisors.initializer import *
 from .base import DummySupervisor
 
 
@@ -37,6 +36,8 @@ def test_creation(mocker):
     assert mocked_options.called
     assert mocked_parser.called
     # test instances
+    assert supv.sockets is None
+    assert supv.external_publisher is None
     assert isinstance(supv.options, SupvisorsOptions)
     assert mocked_srv_options.realize.called
     assert isinstance(supv.logger, Logger)
@@ -51,18 +52,17 @@ def test_creation(mocker):
     assert isinstance(supv.listener, SupervisorListener)
 
 
-def test_create_logger(mocker):
+def test_create_logger():
     """ Test the create_logger method. """
-    mocker.patch('supvisors.initializer.SupvisorsServerOptions')
     # create Supvisors instance
     supervisor = DummySupervisor()
-    supvisors = Supvisors(supervisor)
     # test AUTO logfile
-    supvisors.options.logfile = Automatic
-    assert supvisors.create_logger(supervisor) is supervisor.options.logger
+    logger_config = get_logger_configuration()
+    logger_config['logfile'] = Automatic
+    assert create_logger(supervisor, logger_config) is supervisor.options.logger
     # test defined logfile
-    supvisors.options.logfile = '/tmp/dummy.log'
-    logger = supvisors.create_logger(supervisor)
+    logger_config['logfile'] = '/tmp/dummy.log'
+    logger = create_logger(supervisor, logger_config)
     assert logger is not supervisor.options.logger
 
 
@@ -70,12 +70,10 @@ def test_identifier_exception(mocker):
     """ Test the values set at construction. """
     mocker.patch('supvisors.initializer.SupvisorsServerOptions')
     mocker.patch('supvisors.initializer.SupvisorsMapper.configure', side_effect=ValueError)
-    # patches Faults codes
-    setattr(Faults, 'SUPVISORS_CONF_ERROR', 777)
     # create Supvisors instance
     supervisord_instance = DummySupervisor()
     # test that local node exception raises a failure to Supervisor
-    with pytest.raises(RPCError):
+    with pytest.raises(ValueError):
         Supvisors(supervisord_instance)
 
 
