@@ -18,15 +18,44 @@
 # ======================================================================
 
 import re
-
-from supervisor.web import OKView, TailView
 from unittest.mock import call
 
-from supvisors.plugin import *
+from supervisor.web import OKView, TailView
 
+from supvisors.plugin import *
 from .base import DummySupervisor
 
 
+def test_expand_faults():
+    """ Test the expand_faults function. """
+    expand_faults()
+    assert SupvisorsFaults.SUPVISORS_CONF_ERROR.value == Faults.SUPVISORS_CONF_ERROR
+    assert SupvisorsFaults.BAD_SUPVISORS_STATE.value == Faults.BAD_SUPVISORS_STATE
+    assert SupvisorsFaults.NOT_MANAGED.value == Faults.NOT_MANAGED
+    assert SupvisorsFaults.DISABLED.value == Faults.DISABLED
+
+
+def test_patch_591():
+    """ Test the patch_591 function. """
+    # check initial context
+    assert not hasattr(SupervisorNamespaceRPCInterface, '_startProcess')
+    assert not hasattr(Subprocess, '_spawn')
+    ref_startProcess = SupervisorNamespaceRPCInterface.startProcess
+    ref_spawn = Subprocess.spawn
+    # check monkeypatch
+    patch_591()
+    assert SupervisorNamespaceRPCInterface._startProcess is ref_startProcess
+    assert Subprocess._spawn is ref_spawn
+    assert SupervisorNamespaceRPCInterface.startProcess is startProcess
+    assert Subprocess.spawn is spawn
+    # check again monkeypatch to ensure that Supvisors patches do not override renamed Supervisor functions
+    patch_591()
+    assert SupervisorNamespaceRPCInterface._startProcess is ref_startProcess
+    assert Subprocess._spawn is ref_spawn
+    assert SupervisorNamespaceRPCInterface.startProcess is startProcess
+    assert Subprocess.spawn is spawn
+
+
 def test_update_views():
     """ Test the update_views function. """
     # update Supervisor views
@@ -69,6 +98,8 @@ def test_update_views():
 
 def test_make_rpc(mocker):
     """ Test the make_supvisors_rpcinterface function. """
+    mocked_expand = mocker.patch('supvisors.plugin.expand_faults')
+    mocked_591 = mocker.patch('supvisors.plugin.patch_591')
     mocked_views = mocker.patch('supvisors.plugin.update_views')
     mocker.patch('supvisors.plugin.Supvisors', return_value='a Supvisors instance')
     mocked_rpc = mocker.patch('supvisors.plugin.RPCInterface')
@@ -79,7 +110,9 @@ def test_make_rpc(mocker):
     make_supvisors_rpcinterface(supervisord)
     # test the calls to previous functions
     assert supervisord.supvisors == 'a Supvisors instance'
+    assert mocked_expand.call_args_list == [call()]
     assert mocked_rpc.call_args_list == [call(supervisord.supvisors)]
+    assert mocked_591.call_args_list == [call()]
     assert mocked_views.call_args_list == [call()]
-    # test inclusion of Supvisors into Supervisor
+    # test monkeypatch of Supervisor cleanup_fds
     assert ServerOptions.cleanup_fds is not cleanup
