@@ -17,19 +17,38 @@
 # limitations under the License.
 # ======================================================================
 
-import pytest
-
-from supervisor.xmlrpc import Faults
 from unittest.mock import call, Mock
 
+import pytest
+from supervisor.rpcinterface import SupervisorNamespaceRPCInterface
+from supervisor.supervisorctl import DefaultControllerPlugin
+from supervisor.xmlrpc import Faults
+
 from supvisors.supvisorsctl import *
+from supvisors.ttypes import SupvisorsFaults
 
 
 @pytest.fixture
-def controller():
+def remote_proxy(mocker):
+    """ Patch the remote proxy getter. """
+
+    def mock_server_proxy(uri, transport):
+        return Mock(uri=uri, transport=transport,
+                    supervisor=Mock(spec=SupervisorNamespaceRPCInterface),
+                    supvisors=Mock(spec=RPCInterface))
+
+    return mocker.patch('supvisors.supvisorsctl.xmlrpclib.ServerProxy', side_effect=mock_server_proxy)
+
+
+@pytest.fixture
+def controller(remote_proxy):
+    """ Patch the controller instance. """
     controller = Mock(spec=Controller, exitstatus=LSBInitExitStatuses.SUCCESS)
     controller.get_server_proxy.return_value = Mock(spec=RPCInterface)
-    controller.options = Mock(serverurl='dummy_url')
+    controller.options = Mock(username='cliche',
+                              password='p@$$w0rd',
+                              serverurl='dummy_url',
+                              plugins=[DefaultControllerPlugin(controller)])
     return controller
 
 
@@ -217,6 +236,35 @@ def _check_stop_process_command(controller, mocked_check, mocked_info, mocked_rp
     _check_output_error(controller, True)
 
 
+def test_creation(controller, plugin):
+    """ Test the creation of the Supvisors ControllerPlugin and the _startresult patch applied to the Supervisor
+    DefaultControllerPlugin. """
+    assert plugin.ctl is controller
+    # check that the Supervisor Faults have been expanded
+    assert Faults.DISABLED == SupvisorsFaults.DISABLED.value
+    # check that the Supervisor plugin has been patched and test the patch
+    supervisor_plugin = controller.options.plugins[0]
+    result = {'group': 'dummy_group', 'name': 'dummy_process', 'status': Faults.DISABLED}
+    assert supervisor_plugin._startresult(result) == 'dummy_group:dummy_process: ERROR disabled'
+    result['status'] = Faults.SUCCESS
+    assert supervisor_plugin._startresult(result) == 'dummy_group:dummy_process: started'
+
+
+def test_get_server_proxy(remote_proxy, plugin):
+    """ Test the access to any remote proxy. """
+    # test the proxy
+    assert SupervisorNamespaceRPCInterface == plugin.get_server_proxy('http://localhost:9000', 'supervisor')._spec_class
+    assert RPCInterface == plugin.get_server_proxy('http://localhost:9000', 'supvisors')._spec_class
+    assert remote_proxy.call_count == 2
+    for called in remote_proxy.call_args_list:
+        args, kwargs = called
+        assert args[0] == 'http://127.0.0.1'
+        assert isinstance(args[1], xmlrpc.SupervisorTransport)
+        assert args[1].username == 'cliche'
+        assert args[1].password == 'p@$$w0rd'
+        assert args[1].serverurl == 'http://localhost:9000'
+
+
 def test_supvisors(controller, plugin):
     """ Test the access to Supvisors proxy. """
     # test the proxy
@@ -224,6 +272,23 @@ def test_supvisors(controller, plugin):
     assert controller.get_server_proxy.call_args_list == [call('supvisors')]
 
 
+def test_get_running_instances(controller, plugin, mocked_check):
+    """ Test the get_running_instances request. """
+    mocked_rpc = plugin.supvisors().get_all_instances_info
+    mocked_rpc.return_value = [{'identifier': 'first', 'node_name': '10.0.0.1', 'port': 30000, 'statecode': 2},
+                               {'identifier': 'second', 'node_name': '10.0.0.2', 'port': 60000, 'statecode': 0}]
+    # test request
+    assert plugin.get_running_instances() == {'first': 'http://10.0.0.1:30000'}
+    # test output (with no error)
+    assert not controller.output.called
+    # test request error
+    mocked_rpc.side_effect = xmlrpclib.Fault(0, 'error')
+    assert plugin.get_running_instances() == {}
+    mocked_rpc.side_effect = None
+    # test output (with error)
+    _check_output_error(controller, True)
+
+
 def test_sversion(controller, plugin, mocked_check):
     """ Test the sversion request. """
     mocked_rpc = plugin.supvisors().get_api_version
@@ -246,7 +311,8 @@ def test_strategies(controller, plugin, mocked_check):
 def test_sstate(controller, plugin, mocked_check):
     """ Test the sstate request. """
     mocked_rpc = plugin.supvisors().get_supvisors_state
-    mocked_rpc.return_value = {'statecode': 10, 'statename': 'running'}
+    mocked_rpc.return_value = {'fsm_statecode': 10, 'fsm_statename': 'running',
+                               'starting_jobs': [], 'stopping_jobs': ['10.0.0.1', 'test']}
     _check_call(controller, mocked_check, mocked_rpc, plugin.help_sstate, plugin.do_sstate, '', [call()])
 
 
@@ -254,9 +320,11 @@ def test_instance_status(controller, plugin, mocked_check):
     """ Test the instance_status request. """
     mocked_rpc = plugin.supvisors().get_all_instances_info
     mocked_rpc.return_value = [{'identifier': '10.0.0.1', 'node_name': '10.0.0.1', 'port': 60000,
-                                'statename': 'running', 'loading': 10, 'local_time': 1500, 'sequence_counter': 12},
+                                'statename': 'running', 'loading': 10, 'local_time': 1500, 'sequence_counter': 12,
+                                'fsm_statename': 'OPERATION', 'starting_jobs': True, 'stopping_jobs': False},
                                {'identifier': '10.0.0.2', 'node_name': '10.0.0.2', 'port': 60000,
-                                'statename': 'stopped', 'loading': 0, 'local_time': 100, 'sequence_counter': 15}]
+                                'statename': 'stopped', 'loading': 0, 'local_time': 100, 'sequence_counter': 15,
+                                'fsm_statename': 'CONCILATION', 'starting_jobs': False, 'stopping_jobs': True}]
     _check_call(controller, mocked_check, mocked_rpc,  plugin.help_instance_status, plugin.do_instance_status,
                 '', [call()])
     _check_call(controller, mocked_check, mocked_rpc, plugin.help_instance_status, plugin.do_instance_status,
@@ -309,10 +377,10 @@ def test_local_status(controller, plugin, mocked_check):
     mocked_rpc = plugin.supvisors().get_all_local_process_info
     mocked_rpc.return_value = [{'group': 'appli_1', 'name': 'proc_1',
                                 'state': 20, 'start': 1234, 'now': 4321, 'pid': 14725,
-                                'extra_args': '-x dummy'},
+                                'extra_args': '-x dummy', 'disabled': False},
                                {'group': 'appli_2', 'name': 'proc_3',
                                 'state': 0, 'start': 0, 'now': 0, 'pid': 0,
-                                'extra_args': ''}]
+                                'extra_args': '', 'disabled': True}]
     _check_call(controller, mocked_check, mocked_rpc, plugin.help_local_status, plugin.do_local_status,
                 '', [call()])
     _check_call(controller, mocked_check, mocked_rpc, plugin.help_local_status, plugin.do_local_status,
@@ -328,11 +396,11 @@ def test_application_rules(controller, plugin, mocked_check):
     mocked_appli.return_value = [{'application_name': 'appli_1'}, {'application_name': 'appli_2'},
                                  {'application_name': 'appli_3'}]
     mocked_rpc = plugin.supvisors().get_application_rules
-    returned_rules = [{'application_name': 'appli_1', 'managed': True, 'distributed': True,
+    returned_rules = [{'application_name': 'appli_1', 'managed': True, 'distribution': 'SINGLE_NODE',
                        'start_sequence': 2, 'stop_sequence': 3, 'starting_strategy': 'CONFIG',
                        'starting_failure_strategy': 'ABORT', 'running_failure_strategy': 'CONTINUE'},
                       {'application_name': 'appli_2', 'managed': True,
-                       'distributed': False, 'addresses': ['10.0.0.1', '10.0.0.2', '10.0.0.3'],
+                       'distribution': 'ALL_INSTANCES', 'addresses': ['10.0.0.1', '10.0.0.2', '10.0.0.3'],
                        'start_sequence': 1, 'stop_sequence': 0, 'starting_strategy': 'LESS_LOADED',
                        'starting_failure_strategy': 'CONTINUE', 'running_failure_strategy': 'RESTART_APPLICATION'},
                       {'application_name': 'appli_3', 'managed': False}]
@@ -442,14 +510,78 @@ def test_stop_application(controller, plugin, mocked_check):
 
 def test_start_args(controller, plugin, mocked_check):
     """ Test the start_args request. """
+    # test request to start process without extra arguments
     plugin.do_start_args('')
     _check_output_error(controller, True)
     assert mocked_check.call_args_list == [call()]
     mocked_check.reset_mock()
-    # test request to start process without extra arguments
+    # test request to start process with extra arguments
     mocked_rpc = plugin.supvisors().start_args
     _check_call(controller, mocked_check, mocked_rpc, plugin.help_start_args, plugin.do_start_args,
-                'proc MOST_LOADED all', [call('proc', 'MOST_LOADED all')])
+                'proc -x 5', [call('proc', '-x 5')])
+
+
+def test_all_start(mocker, controller, plugin, mocked_check):
+    """ Test the all_start request. """
+    mocked_get_instances = mocker.patch.object(plugin, 'get_running_instances')
+    mocked_rpc = Mock(**{'startProcess.return_value': True})
+    mocked_get_proxy = mocker.patch.object(plugin, 'get_server_proxy', return_value=mocked_rpc)
+    # test request to start process without arguments
+    plugin.do_all_start('')
+    _check_output_error(controller, True)
+    assert mocked_check.call_args_list == [call()]
+    mocked_check.reset_mock()
+    # test without running instances (doesn't really make sense)
+    mocked_get_instances.return_value = {}
+    plugin.do_all_start('appli_1:proc_1')
+    _check_output_error(controller, True)
+    assert mocked_check.call_args_list == [call()]
+    mocked_check.reset_mock()
+    assert not mocked_get_proxy.called
+    assert not mocked_rpc.startProcess.called
+    # test with running instances
+    mocked_get_instances.return_value = {'node_1': 'http://10.0.0.1:30000', 'node_2': 'http://10.0.0.2:31000'}
+    _check_call(controller, mocked_check, mocked_rpc.startProcess, plugin.help_all_start, plugin.do_all_start,
+                'appli_1:proc_1', [call('appli_1:proc_1', False), call('appli_1:proc_1', False)])
+    # get_server_proxy is called 2x2 times, without error and with error (which is set on startProcess)
+    assert mocked_get_proxy.call_args_list == [call('http://10.0.0.1:30000', 'supervisor'),
+                                               call('http://10.0.0.2:31000', 'supervisor'),
+                                               call('http://10.0.0.1:30000', 'supervisor'),
+                                               call('http://10.0.0.2:31000', 'supervisor')]
+
+
+def test_all_start_args(mocker, controller, plugin, mocked_check):
+    """ TODO: Test the all_start_args request. """
+    mocked_get_instances = mocker.patch.object(plugin, 'get_running_instances')
+    mocked_rpc = Mock(**{'start_args.return_value': True})
+    mocked_get_proxy = mocker.patch.object(plugin, 'get_server_proxy', return_value=mocked_rpc)
+    # test request to start process without arguments
+    plugin.do_all_start_args('')
+    _check_output_error(controller, True)
+    assert mocked_check.call_args_list == [call()]
+    mocked_check.reset_mock()
+    # test request to start process without extra arguments
+    plugin.do_all_start_args('appli_1:proc_1')
+    _check_output_error(controller, True)
+    assert mocked_check.call_args_list == [call()]
+    mocked_check.reset_mock()
+    # test without running instances (doesn't really make sense)
+    mocked_get_instances.return_value = {}
+    plugin.do_all_start_args('appli_1:proc_1 -x 5')
+    _check_output_error(controller, True)
+    assert mocked_check.call_args_list == [call()]
+    mocked_check.reset_mock()
+    assert not mocked_get_proxy.called
+    assert not mocked_rpc.start_args.called
+    # test with running instances
+    mocked_get_instances.return_value = {'node_1': 'http://10.0.0.1:30000', 'node_2': 'http://10.0.0.2:31000'}
+    _check_call(controller, mocked_check, mocked_rpc.start_args, plugin.help_all_start_args, plugin.do_all_start_args,
+                'appli_1:proc_1 -x 5 ', [call('appli_1:proc_1', '-x 5', False), call('appli_1:proc_1', '-x 5', False)])
+    # get_server_proxy is called 2x2 times, without error and with error (which is set on startProcess)
+    assert mocked_get_proxy.call_args_list == [call('http://10.0.0.1:30000', 'supvisors'),
+                                               call('http://10.0.0.2:31000', 'supvisors'),
+                                               call('http://10.0.0.1:30000', 'supvisors'),
+                                               call('http://10.0.0.2:31000', 'supvisors')]
 
 
 def test_start_process(controller, plugin, mocked_check):
@@ -462,6 +594,56 @@ def test_start_process(controller, plugin, mocked_check):
                                  ('appli_1:proc_1', 'appli_2:proc_3'))
 
 
+def test_start_any_process(controller, plugin, mocked_check):
+    """ Test the start_any_process request. """
+    # test the request using few arguments
+    plugin.do_start_any_process('')
+    _check_output_error(controller, True)
+    assert mocked_check.call_args_list == [call()]
+    mocked_check.reset_mock()
+    plugin.do_start_any_process('CONFIG')
+    _check_output_error(controller, True)
+    assert mocked_check.call_args_list == [call()]
+    mocked_check.reset_mock()
+    # test the request using unknown strategy
+    plugin.do_start_any_process('strategy regex')
+    _check_output_error(controller, True)
+    assert mocked_check.call_args_list == [call()]
+    mocked_check.reset_mock()
+    # test request to start the process
+    mocked_rpc = plugin.supvisors().start_any_process
+    _check_call(controller, mocked_check, mocked_rpc,
+                plugin.help_start_any_process, plugin.do_start_any_process,
+                'LESS_LOADED :x [abc]', [call(1, ':x'), call(1, '[abc]')])
+
+
+def test_start_any_process_args(controller, plugin, mocked_check):
+    """ Test the start_any_process_args request. """
+    # test the request using few arguments
+    plugin.do_start_any_process_args('')
+    _check_output_error(controller, True)
+    assert mocked_check.call_args_list == [call()]
+    mocked_check.reset_mock()
+    plugin.do_start_any_process_args('CONFIG')
+    _check_output_error(controller, True)
+    assert mocked_check.call_args_list == [call()]
+    mocked_check.reset_mock()
+    plugin.do_start_any_process_args('CONFIG regex')
+    _check_output_error(controller, True)
+    assert mocked_check.call_args_list == [call()]
+    mocked_check.reset_mock()
+    # test the request using unknown strategy
+    plugin.do_start_any_process_args('strategy regex a list of arguments')
+    _check_output_error(controller, True)
+    assert mocked_check.call_args_list == [call()]
+    mocked_check.reset_mock()
+    # test request to start the process
+    mocked_rpc = plugin.supvisors().start_any_process
+    _check_call(controller, mocked_check, mocked_rpc,
+                plugin.help_start_any_process_args, plugin.do_start_any_process_args,
+                'LESS_LOADED :x  a list of arguments', [call(1, ':x', 'a list of arguments')])
+
+
 def test_restart_process(controller, plugin, mocked_check):
     """ Test the restart_process request. """
     mocked_info = plugin.supvisors().get_all_process_info
@@ -511,7 +693,7 @@ def test_stop_process(controller, plugin, mocked_check):
 
 
 def test_update_numprocs(controller, plugin, mocked_check):
-    """ Test the conciliate request. """
+    """ Test the update_numprocs request. """
     plugin.do_update_numprocs('')
     _check_output_error(controller, True)
     assert mocked_check.call_args_list == [call()]
@@ -532,6 +714,30 @@ def test_update_numprocs(controller, plugin, mocked_check):
                 'dummy_process 2', [call('dummy_process', 2)])
 
 
+def test_enable(controller, plugin, mocked_check):
+    """ Test the enable request. """
+    plugin.do_enable('')
+    _check_output_error(controller, True)
+    assert mocked_check.call_args_list == [call()]
+    mocked_check.reset_mock()
+    # test help and request
+    mocked_rpc = plugin.supvisors().enable
+    _check_call(controller, mocked_check, mocked_rpc, plugin.help_enable, plugin.do_enable,
+                'dummy_process', [call('dummy_process')])
+
+
+def test_disable(controller, plugin, mocked_check):
+    """ Test the disable request. """
+    plugin.do_disable('')
+    _check_output_error(controller, True)
+    assert mocked_check.call_args_list == [call()]
+    mocked_check.reset_mock()
+    # test help and request
+    mocked_rpc = plugin.supvisors().disable
+    _check_call(controller, mocked_check, mocked_rpc, plugin.help_disable, plugin.do_disable,
+                'dummy_process', [call('dummy_process')])
+
+
 def test_conciliate(controller, plugin, mocked_check):
     """ Test the conciliate request. """
     plugin.do_conciliate('')
