@@ -17,12 +17,13 @@
 # limitations under the License.
 # ======================================================================
 
+import re
+from unittest.mock import call, patch, Mock
+
 import pytest
 
 from supvisors.viewcontext import *
-from unittest.mock import call, patch, Mock
-
-from .base import DummyHttpContext, DummyOptions
+from .base import DummyHttpContext
 
 url_attr_template = r'(.+=.+)'
 
@@ -38,16 +39,6 @@ def http_context(supvisors):
 @pytest.fixture
 def ctx(mocker, http_context):
     """ Fixture for the instance to test. """
-    mocker.patch.object(http_context.supervisord.supvisors.context, 'get_all_namespecs', return_value={})
-    return ViewContext(http_context)
-
-
-@pytest.fixture
-def ctx_no_stats(mocker, http_context):
-    """ Fixture for the instance to test. """
-    supvisors = http_context.supervisord.supvisors
-    mocker.patch.object(supvisors.context, 'get_all_namespecs', return_value={})
-    supvisors.options.stats_enabled = False
     return ViewContext(http_context)
 
 
@@ -66,20 +57,6 @@ def test_init(http_context, ctx):
     assert not ctx.redirect
 
 
-def test_init_no_stats(http_context, ctx_no_stats):
-    """ Test the values set at ViewContext construction. """
-    assert ctx_no_stats.http_context is http_context
-    assert ctx_no_stats.supvisors is http_context.supervisord.supvisors
-    assert ctx_no_stats.local_identifier == ctx_no_stats.supvisors.supvisors_mapper.local_identifier
-    assert ctx_no_stats.parameters == {'ident': '10.0.0.4', 'namespec': None, 'appliname': None,
-                                       'processname': None, 'auto': False, 'strategy': 'CONFIG', 'shex': ''}
-    # errors must be set due to dummy values
-    assert isinstance(ctx_no_stats.store_message, tuple)
-    assert len(ctx_no_stats.store_message) == 2
-    assert ctx_no_stats.store_message[0] == 'erro'
-    assert not ctx_no_stats.redirect
-
-
 def test_get_action(ctx):
     """ Test the ViewContext.get_action method. """
     assert ctx.get_action() == 'test'
@@ -261,8 +238,8 @@ def test_update_period(mocker, ctx):
     mocked_update = mocker.patch('supvisors.viewcontext.ViewContext._update_integer')
     # test method with statistics enabled
     ctx.update_period()
-    assert mocked_update.call_args_list == [call(PERIOD, DummyOptions().stats_periods,
-                                                 DummyOptions().stats_periods[0])]
+    assert mocked_update.call_args_list == [call(PERIOD, ctx.supvisors.options.stats_periods,
+                                                 ctx.supvisors.options.stats_periods[0])]
     mocker.resetall()
     # test method with statistics disabled
     ctx.supvisors.options.stats_periods = []
@@ -348,20 +325,21 @@ def test_update_process_name(mocker, ctx):
     assert ctx.parameters[PROCESS] is None
 
 
-def test_update_namespec(ctx):
+def test_update_namespec(mocker, ctx):
     """ Test the ViewContext.update_namespec method. """
     # reset parameter because called in constructor
     del ctx.parameters[NAMESPEC]
-    # test call with valid parameter
-    ctx.http_context.supervisord.supvisors.context.get_all_namespecs.return_value = ['abc', 'dummy_proc']
+    # test call with no value
     ctx.update_namespec()
-    assert ctx.parameters[NAMESPEC] == 'dummy_proc'
-    # reset parameter
-    del ctx.parameters[NAMESPEC]
+    assert ctx.parameters[NAMESPEC] is None
     # test call with invalid value
     ctx.http_context.form[NAMESPEC] = 'any_proc'
     ctx.update_namespec()
     assert ctx.parameters[NAMESPEC] is None
+    # test call with valid parameter
+    mocker.patch.object(ctx.http_context.supervisord.supvisors.context, 'is_namespec', return_value=True)
+    ctx.update_namespec()
+    assert ctx.parameters[NAMESPEC] == 'any_proc'
 
 
 def test_update_cpu_id(mocker, ctx):
