@@ -17,13 +17,12 @@
 # limitations under the License.
 # ======================================================================
 
-import pytest
 import time
-
 from unittest.mock import call, Mock
 
+import pytest
 from supervisor.http import NOT_DONE_YET
-from supervisor.states import SupervisorStates, RUNNING_STATES, STOPPED_STATES
+from supervisor.states import SupervisorStates, ProcessStates
 
 from supvisors.rpcinterface import API_VERSION
 from supvisors.ttypes import ApplicationStates, StartingStrategies, SupvisorsStates, SupvisorsInstanceStates
@@ -31,8 +30,8 @@ from supvisors.viewcontext import AUTO, PERIOD, PROCESS, ViewContext
 from supvisors.viewhandler import ViewHandler
 from supvisors.viewimage import process_cpu_img, process_mem_img
 from supvisors.webutils import SUPVISORS_PAGE, MASTER_SYMBOL
-
 from .base import DummyHttpContext
+from .conftest import create_element, create_application
 
 
 @pytest.fixture
@@ -64,6 +63,7 @@ def test_init(http_context, handler):
     assert handler.supvisors is http_context.supervisord.supvisors
     assert handler.sup_ctx is http_context.supervisord.supvisors.context
     assert handler.local_identifier == handler.supvisors.supvisors_mapper.local_identifier
+    assert handler.has_statistics
     assert handler.view_ctx is None
 
 
@@ -266,126 +266,127 @@ def test_write_nav_instances_silent_instance(handler):
 
 def test_write_nav_instances_running_instance(handler):
     """ Test the write_nav_instances method using a RUNNING instance. """
+    # set context
+    status = handler.sup_ctx.instances['10.0.0.1']
+    status._state = SupvisorsInstanceStates.RUNNING
     # patch the meld elements
-    href_elt = Mock(attrib={})
-    address_elt = Mock(attrib={}, **{'findmeld.return_value': href_elt})
-    mocked_mid = Mock(**{'repeat.return_value': [(address_elt, '10.0.0.1')]})
-    mocked_root = Mock(**{'findmeld.return_value': mocked_mid})
+    instance_a_mid = create_element()
+    instance_elt = create_element({'instance_a_mid': instance_a_mid})
+    instance_li_mid = create_element()
+    instance_li_mid.repeat.return_value = [(instance_elt, '10.0.0.1')]
+    mocked_root = create_element({'instance_li_mid': instance_li_mid})
     # test call with address status set in context, RUNNING, different from parameter and not MASTER
     handler.view_ctx = Mock(**{'format_url.return_value': 'an url'})
-    handler.sup_ctx.instances['10.0.0.1'] = Mock(state=SupvisorsInstanceStates.RUNNING,
-                                                 **{'state_string.return_value': 'running'})
     handler.write_nav_instances(mocked_root, '10.0.0.2')
     assert mocked_root.findmeld.call_args_list == [call('instance_li_mid')]
-    assert mocked_mid.repeat.call_args_list == [call(list(handler.supvisors.supvisors_mapper.instances.keys()))]
-    assert address_elt.attrib['class'] == 'RUNNING'
-    assert address_elt.findmeld.call_args_list == [call('instance_a_mid')]
+    assert instance_li_mid.repeat.call_args_list == [call(list(handler.supvisors.supvisors_mapper.instances.keys()))]
+    assert instance_elt.attrib['class'] == 'RUNNING'
+    assert instance_elt.findmeld.call_args_list == [call('instance_a_mid')]
     assert handler.view_ctx.format_url.call_args_list == [call('10.0.0.1', 'proc_instance.html')]
-    assert href_elt.attributes.call_args_list == [call(href='an url')]
-    assert href_elt.attrib['class'] == 'on'
-    assert href_elt.content.call_args_list == [call('10.0.0.1')]
-    mocked_root.findmeld.reset_mock()
-    mocked_mid.repeat.reset_mock()
-    address_elt.findmeld.reset_mock()
+    assert instance_a_mid.attributes.call_args_list == [call(href='an url')]
+    assert instance_a_mid.attrib['class'] == 'on'
+    assert instance_a_mid.content.call_args_list == [call('10.0.0.1')]
+    instance_elt.reset_all()
+    mocked_root.reset_all()
     handler.view_ctx.format_url.reset_mock()
-    href_elt.attributes.reset_mock()
-    href_elt.content.reset_mock()
     # test call with address status set in context, RUNNING, identical to parameter and MASTER
+    status.state_modes.starting_jobs = True
     handler.sup_ctx.master_identifier = '10.0.0.1'
     handler.write_nav_instances(mocked_root, '10.0.0.1')
     assert mocked_root.findmeld.call_args_list == [call('instance_li_mid')]
-    assert mocked_mid.repeat.call_args_list == [call(list(handler.supvisors.supvisors_mapper.instances.keys()))]
-    assert address_elt.attrib['class'] == 'RUNNING active'
-    assert address_elt.findmeld.call_args_list == [call('instance_a_mid')]
+    assert instance_li_mid.repeat.call_args_list == [call(list(handler.supvisors.supvisors_mapper.instances.keys()))]
+    assert instance_elt.attrib['class'] == 'RUNNING active'
+    assert instance_elt.findmeld.call_args_list == [call('instance_a_mid')]
     assert handler.view_ctx.format_url.call_args_list == [call('10.0.0.1', 'proc_instance.html')]
-    assert href_elt.attributes.call_args_list == [call(href='an url')]
-    assert href_elt.attrib['class'] == 'on'
-    assert href_elt.content.call_args_list == [call(f'{MASTER_SYMBOL} 10.0.0.1')]
+    assert instance_a_mid.attributes.call_args_list == [call(href='an url')]
+    assert instance_a_mid.attrib['class'] == 'blink on'
+    assert instance_a_mid.content.call_args_list == [call(f'{MASTER_SYMBOL} 10.0.0.1')]
 
 
 def test_write_nav_applications_initialization(handler):
     """ Test the write_nav_applications method with Supvisors in its INITIALIZATION state. """
     handler.supvisors.fsm.state = SupvisorsStates.INITIALIZATION
+    dummy_appli = create_application('dummy_appli', handler.supvisors)
+    dummy_appli._state = ApplicationStates.RUNNING
+    handler.supvisors.starter.get_application_job_names.return_value = set()
+    handler.supvisors.stopper.get_application_job_names.return_value = set()
     # patch the meld elements
-    href_elt = Mock(attrib={})
-    appli_elt = Mock(attrib={}, **{'findmeld.return_value': href_elt})
-    mocked_appli = Mock(application_name='dummy_appli', major_failure=False, minor_failure=False,
-                        state=ApplicationStates.RUNNING)
-    mocked_li_mid = Mock(**{'repeat.return_value': [(appli_elt, mocked_appli)]})
-    mocked_h_mid = Mock(attrib={})
-    mocked_root = Mock(**{'findmeld.side_effect': [mocked_li_mid, mocked_li_mid, mocked_h_mid]})
+    appli_a_mid = create_element()
+    appli_elt = create_element({'appli_a_mid': appli_a_mid})
+    appli_li_mid = create_element()
+    appli_li_mid.repeat.return_value = [(appli_elt, dummy_appli)]
+    appli_h_mid = create_element()
+    mocked_root = create_element({'appli_li_mid': appli_li_mid, 'appli_h_mid': appli_h_mid})
     # test call with application name different from parameter
     handler.view_ctx = Mock(**{'format_url.return_value': 'an url'})
     handler.write_nav_applications(mocked_root, 'dumb_appli')
     assert mocked_root.findmeld.call_args_list == [call('appli_li_mid')]
     assert appli_elt.attrib['class'] == 'RUNNING'
     assert appli_elt.findmeld.call_args_list == [call('appli_a_mid')]
-    assert href_elt.attrib['class'] == 'off'
+    assert appli_a_mid.attrib['class'] == 'off'
     assert handler.view_ctx.format_url.call_args_list == []
-    assert href_elt.attributes.call_args_list == []
-    assert href_elt.content.call_args_list == [call('dummy_appli')]
-    assert mocked_h_mid.attrib == {}
-    mocked_root.findmeld.reset_mock()
-    mocked_li_mid.repeat.reset_mock()
-    appli_elt.findmeld.reset_mock()
-    href_elt.content.reset_mock()
-    del appli_elt.attrib['class']
-    del href_elt.attrib['class']
+    assert appli_a_mid.attributes.call_args_list == []
+    assert appli_a_mid.content.call_args_list == [call('dummy_appli')]
+    assert appli_h_mid.attrib['class'] == ''
+    mocked_root.reset_all()
+    appli_elt.reset_all()
     # test call with application name identical to parameter and add a failure
-    mocked_appli.minor_failure = True
+    handler.supvisors.starter.get_application_job_names.return_value = {'dummy_appli'}
+    dummy_appli.minor_failure = True
     handler.write_nav_applications(mocked_root, 'dummy_appli')
     assert mocked_root.findmeld.call_args_list == [call('appli_li_mid'), call('appli_h_mid')]
     assert appli_elt.attrib['class'] == 'RUNNING active failure'
     assert appli_elt.findmeld.call_args_list == [call('appli_a_mid')]
-    assert href_elt.attrib['class'] == 'off'
+    assert appli_a_mid.attrib['class'] == 'blink off'
     assert handler.view_ctx.format_url.call_args_list == []
-    assert href_elt.attributes.call_args_list == []
-    assert href_elt.content.call_args_list == [call('dummy_appli')]
-    assert mocked_h_mid.attrib['class'] == 'failure'
+    assert appli_a_mid.attributes.call_args_list == []
+    assert appli_a_mid.content.call_args_list == [call('dummy_appli')]
+    assert appli_h_mid.attrib['class'] == 'failure'
 
 
 def test_write_nav_applications_operation(handler):
     """ Test the write_nav_applications method with Supvisors in its OPERATION state. """
     handler.supvisors.fsm.state = SupvisorsStates.OPERATION
+    dummy_appli = create_application('dummy_appli', handler.supvisors)
+    dummy_appli._state = ApplicationStates.RUNNING
+    dummy_appli.rules.starting_strategy = StartingStrategies.LESS_LOADED
+    handler.supvisors.starter.get_application_job_names.return_value = set()
+    handler.supvisors.stopper.get_application_job_names.return_value = set()
     # patch the meld elements
-    href_elt = Mock(attrib={})
-    appli_elt = Mock(attrib={}, **{'findmeld.return_value': href_elt})
-    mocked_appli = Mock(application_name='dummy_appli', major_failure=False, minor_failure=False,
-                        state=ApplicationStates.RUNNING, rules=Mock(starting_strategy=StartingStrategies.LESS_LOADED))
-    mocked_li_mid = Mock(**{'repeat.return_value': [(appli_elt, mocked_appli)]})
-    mocked_h_mid = Mock(attrib={})
-    mocked_root = Mock(**{'findmeld.side_effect': [mocked_li_mid, mocked_h_mid, mocked_li_mid]})
+    appli_a_mid = create_element()
+    appli_elt = create_element({'appli_a_mid': appli_a_mid})
+    appli_li_mid = create_element()
+    appli_li_mid.repeat.return_value = [(appli_elt, dummy_appli)]
+    appli_h_mid = create_element()
+    mocked_root = create_element({'appli_li_mid': appli_li_mid, 'appli_h_mid': appli_h_mid})
     # test call with application name different from parameter and failure
-    mocked_appli.major_failure = True
+    dummy_appli.major_failure = True
     handler.view_ctx = Mock(**{'format_url.return_value': 'an url'})
     handler.write_nav_applications(mocked_root, 'dumb_appli')
     assert mocked_root.findmeld.call_args_list == [call('appli_li_mid'), call('appli_h_mid')]
     assert appli_elt.attrib['class'] == 'RUNNING failure'
     assert appli_elt.findmeld.call_args_list == [call('appli_a_mid')]
-    assert href_elt.attrib['class'] == 'on'
+    assert appli_a_mid.attrib['class'] == 'on'
     assert handler.view_ctx.format_url.call_args_list == [call('', 'application.html', appliname='dummy_appli',
                                                                strategy='LESS_LOADED')]
-    assert href_elt.attributes.call_args_list == [call(href='an url')]
-    assert href_elt.content.call_args_list == [call('dummy_appli')]
-    assert mocked_h_mid.attrib['class'] == 'failure'
-    mocked_root.findmeld.reset_mock()
-    appli_elt.findmeld.reset_mock()
+    assert appli_a_mid.attributes.call_args_list == [call(href='an url')]
+    assert appli_a_mid.content.call_args_list == [call('dummy_appli')]
+    assert appli_h_mid.attrib['class'] == 'failure'
     handler.view_ctx.format_url.reset_mock()
-    href_elt.attributes.reset_mock()
-    href_elt.content.reset_mock()
-    del appli_elt.attrib['class']
-    del href_elt.attrib['class']
+    mocked_root.reset_all()
+    appli_elt.reset_all()
     # test call with application name identical to parameter
-    mocked_appli.major_failure = False
+    handler.supvisors.stopper.get_application_job_names.return_value = {'dummy_appli'}
+    dummy_appli.major_failure = False
     handler.write_nav_applications(mocked_root, 'dummy_appli')
     assert mocked_root.findmeld.call_args_list == [call('appli_li_mid')]
     assert appli_elt.attrib['class'] == 'RUNNING active'
     assert appli_elt.findmeld.call_args_list == [call('appli_a_mid')]
-    assert href_elt.attrib['class'] == 'on'
+    assert appli_a_mid.attrib['class'] == 'blink on'
     assert handler.view_ctx.format_url.call_args_list == [call('', 'application.html', appliname='dummy_appli',
                                                                strategy='LESS_LOADED')]
-    assert href_elt.attributes.call_args_list == [call(href='an url')]
-    assert href_elt.content.call_args_list == [call('dummy_appli')]
+    assert appli_a_mid.attributes.call_args_list == [call(href='an url')]
+    assert appli_a_mid.content.call_args_list == [call('dummy_appli')]
 
 
 def test_write_header(handler):
@@ -400,7 +401,7 @@ def test_write_periods(handler):
     mocked_mid = Mock()
     mocked_root = Mock(**{'findmeld.return_value': mocked_mid})
     # test call when statistics are disabled
-    handler.supvisors.options.stats_enabled = False
+    handler.has_statistics = False
     handler.write_periods(mocked_root)
     assert mocked_root.findmeld.call_args_list == [call('period_div_mid')]
     assert mocked_mid.replace.call_args_list == [call('')]
@@ -410,7 +411,7 @@ def test_write_periods(handler):
     mocked_mid = Mock(**{'repeat.return_value': [(period_elt, 5)]})
     mocked_root = Mock(**{'findmeld.return_value': mocked_mid})
     # test call with period selection identical to parameter
-    handler.supvisors.options.stats_enabled = True
+    handler.has_statistics = True
     handler.view_ctx = Mock(parameters={PERIOD: 5}, **{'format_url.return_value': 'an url'})
     handler.write_periods(mocked_root)
     assert mocked_root.findmeld.call_args_list == [call('period_li_mid')]
@@ -519,7 +520,7 @@ def test_write_common_process_cpu(handler):
     tr_elt.findmeld.reset_mock()
     cell_elt.replace.reset_mock()
     # test with statistics disabled
-    handler.supvisors.options.stats_enabled = False
+    handler.has_statistics = False
     handler.write_common_process_cpu(tr_elt, info)
     assert tr_elt.findmeld.call_args_list == [call('pcpu_td_mid')]
     assert cell_elt.deparent.call_args_list == [call()]
@@ -604,7 +605,7 @@ def test_write_common_process_mem(handler):
     tr_elt.findmeld.reset_mock()
     cell_elt.replace.reset_mock()
     # test with statistics disabled
-    handler.supvisors.options.stats_enabled = False
+    handler.has_statistics = False
     handler.write_common_process_mem(tr_elt, info)
     assert tr_elt.findmeld.call_args_list == [call('pmem_td_mid')]
     assert cell_elt.deparent.call_args_list == [call()]
@@ -618,55 +619,74 @@ def test_write_process_start_button(mocker, handler):
     """ Test the write_process_start_button method. """
     mocked_button = mocker.patch('supvisors.viewhandler.ViewHandler._write_process_button')
     handler.page_name = 'My Page'
-    # test call indirection
-    info = {'namespec': 'dummy_proc', 'statecode': 'stopped'}
+    # test call redirection when program is disabled
+    info = {'namespec': 'dummy_proc', 'statecode': ProcessStates.STOPPED, 'startable': False}
     handler.write_process_start_button('elt', info)
-    assert mocked_button.call_args_list == [call('elt', 'start_a_mid', '', 'My Page', 'start', 'dummy_proc',
-                                                 'stopped', STOPPED_STATES)]
+    assert mocked_button.call_args_list == [call('elt', 'start_a_mid', '', 'My Page', 'start', 'dummy_proc', False)]
+    mocked_button.reset_mock()
+    # test call redirection when program is enabled
+    info['startable'] = True
+    handler.write_process_start_button('elt', info)
+    assert mocked_button.call_args_list == [call('elt', 'start_a_mid', '', 'My Page', 'start', 'dummy_proc', True)]
 
 
 def test_write_process_stop_button(mocker, handler):
     """ Test the write_process_stop_button method. """
     mocked_button = mocker.patch('supvisors.viewhandler.ViewHandler._write_process_button')
     handler.page_name = 'My Page'
-    # test call indirection
-    info = {'namespec': 'dummy_proc', 'statecode': 'starting'}
+    # test call redirection
+    info = {'namespec': 'dummy_proc', 'statecode': ProcessStates.STARTING}
     handler.write_process_stop_button('elt', info)
-    assert mocked_button.call_args_list == [call('elt', 'stop_a_mid', '', 'My Page', 'stop', 'dummy_proc',
-                                                 'starting', RUNNING_STATES)]
+    assert mocked_button.call_args_list == [call('elt', 'stop_a_mid', '', 'My Page', 'stop', 'dummy_proc', True)]
 
 
 def test_write_process_restart_button(mocker, handler):
     """ Test the write_process_restart_button method. """
     mocked_button = mocker.patch('supvisors.viewhandler.ViewHandler._write_process_button')
     handler.page_name = 'My Page'
-    # test call indirection
-    info = {'namespec': 'dummy_proc', 'statecode': 'running'}
+    # test call redirection when program is disabled
+    info = {'namespec': 'dummy_proc', 'statecode': ProcessStates.RUNNING, 'startable': False}
     handler.write_process_restart_button('elt', info)
-    assert mocked_button.call_args_list == [call('elt', 'restart_a_mid', '', 'My Page',
-                                                 'restart', 'dummy_proc', 'running', RUNNING_STATES)]
+    assert mocked_button.call_args_list == [call('elt', 'restart_a_mid', '', 'My Page', 'restart', 'dummy_proc', False)]
+    mocked_button.reset_mock()
+    # test call redirection when program is enabled
+    info['startable'] = True
+    handler.write_process_restart_button('elt', info)
+    assert mocked_button.call_args_list == [call('elt', 'restart_a_mid', '', 'My Page', 'restart', 'dummy_proc', True)]
 
 
 def test_write_process_clear_button(mocker, handler):
     """ Test the write_process_clear_button method. """
     mocked_button = mocker.patch('supvisors.viewhandler.ViewHandler._write_process_button')
     handler.page_name = 'My Page'
-    # test call indirection
-    info = {'namespec': 'dummy_proc', 'identifier': '10.0.0.1'}
-    handler.write_process_clear_button('elt', info)
-    assert mocked_button.call_args_list == [call('elt', 'clear_a_mid', '10.0.0.1', 'My Page',
-                                                 'clearlog', 'dummy_proc', '', '')]
+    # test call indirection with log check
+    info = {'namespec': 'dummy_application:dummy_process_1', 'identifier': '10.0.0.1'}
+    handler.write_process_clear_button('elt', info, True)
+    assert mocked_button.call_args_list == [call('elt', 'clear_a_mid', '10.0.0.1', 'My Page', 'clearlog',
+                                                 'dummy_application:dummy_process_1', True)]
+    mocked_button.reset_mock()
+    # test call indirection without log check
+    handler.write_process_clear_button('elt', info, False)
+    assert mocked_button.call_args_list == [call('elt', 'clear_a_mid', '10.0.0.1', 'My Page', 'clearlog',
+                                                 'dummy_application:dummy_process_1', True)]
 
 
 def test_write_process_stdout_button(mocker, handler):
     """ Test the write_process_stdout_button method. """
     mocked_button = mocker.patch('supvisors.viewhandler.ViewHandler._write_process_button')
     handler.page_name = 'My Page'
-    # test call indirection
-    info = {'namespec': 'dummy_proc', 'identifier': '10.0.0.1'}
-    handler.write_process_stdout_button('elt', info)
-    assert mocked_button.call_args_list == [call('elt', 'tailout_a_mid', '10.0.0.1', 'logtail/dummy_proc',
-                                                 '', 'dummy_proc', '', '')]
+    # test call indirection with log check
+    info = {'namespec': 'dummy_application:dummy_process_1', 'identifier': '10.0.0.1'}
+    handler.write_process_stdout_button('elt', info, True)
+    assert mocked_button.call_args_list == [call('elt', 'tailout_a_mid', '10.0.0.1',
+                                                 'logtail/dummy_application%3Adummy_process_1',
+                                                 '', 'dummy_application:dummy_process_1', True)]
+    mocked_button.reset_mock()
+    # test call indirection without log check
+    handler.write_process_stdout_button('elt', info, False)
+    assert mocked_button.call_args_list == [call('elt', 'tailout_a_mid', '10.0.0.1',
+                                                 'logtail/dummy_application%3Adummy_process_1',
+                                                 '', 'dummy_application:dummy_process_1', True)]
 
 
 def test_write_process_stderr_button(mocker, handler):
@@ -674,11 +694,17 @@ def test_write_process_stderr_button(mocker, handler):
     mocked_button = mocker.patch('supvisors.viewhandler.ViewHandler._write_process_button')
     handler.page_name = 'My Page'
     # test call indirection
-    info = {'namespec': 'dummy_proc', 'identifier': '10.0.0.1'}
-    handler.write_process_stderr_button('elt', info)
-    assert mocked_button.call_args_list == [call('elt', 'tailerr_a_mid', '10.0.0.1', 'logtail/dummy_proc/stderr',
-                                                 '', 'dummy_proc', '', '')]
-
+    info = {'namespec': 'dummy_application:dummy_process_1', 'identifier': '10.0.0.1'}
+    handler.write_process_stderr_button('elt', info, True)
+    assert mocked_button.call_args_list == [call('elt', 'tailerr_a_mid', '10.0.0.1',
+                                                 'logtail/dummy_application%3Adummy_process_1/stderr',
+                                                 '', 'dummy_application:dummy_process_1', False)]
+    mocked_button.reset_mock()
+    # test call indirection
+    handler.write_process_stderr_button('elt', info, False)
+    assert mocked_button.call_args_list == [call('elt', 'tailerr_a_mid', '10.0.0.1',
+                                                 'logtail/dummy_application%3Adummy_process_1/stderr',
+                                                 '', 'dummy_application:dummy_process_1', True)]
 
 def test_write_process_button(handler):
     """ Test the _write_process_button method. """
@@ -688,8 +714,7 @@ def test_write_process_button(handler):
     cell_elt = Mock(attrib={'class': ''})
     tr_elt = Mock(**{'findmeld.return_value': cell_elt})
     # test with process state not in expected list
-    handler._write_process_button(tr_elt, 'meld_id', '10.0.0.1', 'index.html', 'action', 'dummy_proc',
-                                  'running', ['stopped', 'stopping'])
+    handler._write_process_button(tr_elt, 'meld_id', '10.0.0.1', 'index.html', 'action', 'dummy_proc', False)
     assert tr_elt.findmeld.call_args_list == [call('meld_id')]
     assert cell_elt.attrib['class'] == 'button off'
     assert not cell_elt.attributes.called
@@ -697,8 +722,7 @@ def test_write_process_button(handler):
     tr_elt.findmeld.reset_mock()
     del cell_elt.attrib['class']
     # test with filled stats on selected process
-    handler._write_process_button(tr_elt, 'meld_id', '10.0.0.1', 'index.html', 'action', 'dummy_proc',
-                                  'running', ['running', 'starting'])
+    handler._write_process_button(tr_elt, 'meld_id', '10.0.0.1', 'index.html', 'action', 'dummy_proc', True)
     assert tr_elt.findmeld.call_args_list == [call('meld_id')]
     assert cell_elt.attrib['class'] == 'button on'
     assert handler.view_ctx.format_url.call_args_list == [call('10.0.0.1', 'index.html', action='action',
@@ -709,14 +733,6 @@ def test_write_process_button(handler):
     handler.view_ctx.format_url.reset_mock()
     cell_elt.attributes.reset_mock()
     del cell_elt.attrib['class']
-    # test with unset namespec
-    handler._write_process_button(tr_elt, 'meld_id', '10.0.0.1', 'index.html', 'action', '',
-                                  'running', ['running', 'starting'])
-    assert tr_elt.findmeld.call_args_list == [call('meld_id')]
-    assert 'class' not in cell_elt.attrib
-    assert not handler.view_ctx.format_url.called
-    assert not cell_elt.attributes.called
-    assert cell_elt.content.call_args_list == [call('')]
 
 
 def test_write_common_process_table(handler):
@@ -729,7 +745,7 @@ def test_write_common_process_table(handler):
                'mem_foot_th_mid': mem_foot_elt, 'cpu_foot_th_mid': cpu_foot_elt, 'total_mid': None}
     root = Mock(attrib={}, **{'findmeld.side_effect': lambda x: mid_map[x]})
     # test with statistics enabled
-    handler.supvisors.options.stats_enabled = True
+    handler.has_statistics = True
     handler.write_common_process_table(root)
     assert not root.findmeld.called
     assert not mem_head_elt.deparent.called
@@ -737,7 +753,7 @@ def test_write_common_process_table(handler):
     assert not cpu_head_elt.deparent.called
     assert not cpu_foot_elt.deparent.called
     # test with statistics enabled
-    handler.supvisors.options.stats_enabled = False
+    handler.has_statistics = False
     handler.write_common_process_table(root)
     assert root.findmeld.call_args_list == [call('mem_head_th_mid'), call('cpu_head_th_mid'),
                                             call('mem_foot_th_mid'), call('cpu_foot_th_mid'), call('total_mid')]
@@ -747,39 +763,42 @@ def test_write_common_process_table(handler):
     assert cpu_foot_elt.deparent.call_args_list == [call()]
 
 
-def test_write_common_status(mocker, handler):
+def test_write_common_state(handler):
+    """ Test the write_common_state method. """
+    # patch the meld elements
+    state_elt = create_element()
+    desc_elt = create_element()
+    tr_elt = create_element({'state_td_mid': state_elt, 'desc_td_mid': desc_elt})
+    # test call on process that never crashed
+    param = {'expected_load': 35, 'statename': 'exited', 'gravity': 'exited', 'disabled': True,
+             'has_crashed': False, 'description': 'something'}
+    handler.write_common_state(tr_elt, param)
+    assert tr_elt.findmeld.call_args_list == [call('state_td_mid'), call('desc_td_mid')]
+    assert state_elt.attrib['class'] == 'exited disabled'
+    assert state_elt.content.call_args_list == [call('exited')]
+    assert desc_elt.content.call_args_list == [call('something')]
+    tr_elt.reset_all()
+    # test call on process that ever crashed
+    param.update({'gravity': 'fatal', 'has_crashed': True, 'disabled': False})
+    handler.write_common_state(tr_elt, param)
+    assert tr_elt.findmeld.call_args_list == [call('state_td_mid'), call('desc_td_mid')]
+    assert state_elt.attrib['class'] == 'fatal crashed'
+    assert state_elt.content.call_args_list == [call('exited')]
+    assert desc_elt.content.call_args_list == [call('something')]
+
+
+def test_write_common_statistics(mocker, handler):
     """ Test the write_common_process_status method. """
     mocked_mem = mocker.patch.object(handler, 'write_common_process_mem')
     mocked_cpu = mocker.patch.object(handler, 'write_common_process_cpu')
     # patch the meld elements
-    state_elt = Mock(attrib={'class': ''})
-    desc_elt = Mock(attrib={'class': ''})
     load_elt = Mock(attrib={'class': ''})
-    mid_map = {'state_td_mid': state_elt, 'desc_td_mid': desc_elt, 'load_td_mid': load_elt}
-    tr_elt = Mock(attrib={}, **{'findmeld.side_effect': lambda x: mid_map[x]})
+    tr_elt = create_element({'load_td_mid': load_elt})
     # test call on process that never crashed
-    param = {'expected_load': 35, 'statename': 'exited', 'gravity': 'exited',
+    param = {'expected_load': 35, 'statename': 'exited', 'gravity': 'exited', 'disabled': True,
              'has_crashed': False, 'description': 'something'}
-    handler.write_common_status(tr_elt, param)
-    assert tr_elt.findmeld.call_args_list == [call('state_td_mid'), call('desc_td_mid'), call('load_td_mid')]
-    assert state_elt.attrib['class'] == 'exited'
-    assert state_elt.content.call_args_list == [call('exited')]
-    assert desc_elt.content.call_args_list == [call('something')]
-    assert load_elt.content.call_args_list == [call('35%')]
-    assert mocked_cpu.call_args_list == [call(tr_elt, param)]
-    assert mocked_mem.call_args_list == [call(tr_elt, param)]
-    state_elt.attrib['class'] = ''
-    mocker.resetall()
-    tr_elt.findmeld.reset_mock()
-    for mid in mid_map.values():
-        mid.reset_mock()
-    # test call on process that ever crashed
-    param.update({'gravity': 'fatal', 'has_crashed': True})
-    handler.write_common_status(tr_elt, param)
-    assert tr_elt.findmeld.call_args_list == [call('state_td_mid'), call('desc_td_mid'), call('load_td_mid')]
-    assert state_elt.attrib['class'] == 'fatal crashed'
-    assert state_elt.content.call_args_list == [call('exited')]
-    assert desc_elt.content.call_args_list == [call('something')]
+    handler.write_common_statistics(tr_elt, param)
+    assert tr_elt.findmeld.call_args_list == [call('load_td_mid')]
     assert load_elt.content.call_args_list == [call('35%')]
     assert mocked_cpu.call_args_list == [call(tr_elt, param)]
     assert mocked_mem.call_args_list == [call(tr_elt, param)]
@@ -793,27 +812,72 @@ def test_write_common_process_status(mocker, handler):
     mocked_restart = mocker.patch('supvisors.viewhandler.ViewHandler.write_process_restart_button')
     mocked_stop = mocker.patch('supvisors.viewhandler.ViewHandler.write_process_stop_button')
     mocked_start = mocker.patch('supvisors.viewhandler.ViewHandler.write_process_start_button')
-    mocked_common = mocker.patch('supvisors.viewhandler.ViewHandler.write_common_status')
+    mocked_state = mocker.patch('supvisors.viewhandler.ViewHandler.write_common_state')
+    mocked_stats = mocker.patch('supvisors.viewhandler.ViewHandler.write_common_statistics')
     # patch the view context
-    handler.view_ctx = Mock(parameters={PROCESS: 'dummy_proc'},
-                            **{'format_url.return_value': 'an url'})
+    handler.view_ctx = Mock(**{'format_url.return_value': 'an url'})
     # patch the meld elements
-    name_elt = Mock(attrib={'class': ''})
-    tr_elt = Mock(attrib={}, **{'findmeld.return_value': name_elt})
-    # test call on selected process
-    param = {'namespec': 'dummy_proc', 'identifier': '10.0.0.1', 'process_name': 'proc'}
+    name_a_elt = create_element()
+    name_td_elt = create_element({'name_a_mid': name_a_elt})
+    tr_elt = create_element({'name_td_mid': name_td_elt})
+    # test call on selected process having a stdout_logfile
+    param = {'namespec': 'dummy_application:dummy_process_1', 'identifier': '10.0.0.1',
+             'process_name': 'dummy_process_1'}
     handler.write_common_process_status(tr_elt, param)
-    assert mocked_common.call_args_list == [call(tr_elt, param)]
-    assert tr_elt.findmeld.call_args_list == [call('name_a_mid')]
-    assert name_elt.content.call_args_list == [call('\u21B3 proc')]
-    assert handler.view_ctx.format_url.call_args_list == [call('10.0.0.1', 'tail.html', processname='dummy_proc')]
-    assert name_elt.attributes.call_args_list == [call(href='an url', target="_blank")]
+    assert mocked_state.call_args_list == [call(tr_elt, param)]
+    assert mocked_stats.call_args_list == [call(tr_elt, param)]
+    assert not name_td_elt.content.called
+    assert name_a_elt.content.call_args_list == [call('\u21B3 dummy_process_1')]
+    assert handler.view_ctx.format_url.call_args_list == [call('10.0.0.1', 'tail.html',
+                                                               processname='dummy_application:dummy_process_1',
+                                                               limit=1024)]
+    assert name_a_elt.attributes.call_args_list == [call(href='an url', target="_blank")]
     assert mocked_start.call_args_list == [call(tr_elt, param)]
     assert mocked_stop.call_args_list == [call(tr_elt, param)]
     assert mocked_restart.call_args_list == [call(tr_elt, param)]
-    assert mocked_clear.call_args_list == [call(tr_elt, param)]
-    assert mocked_stdout.call_args_list == [call(tr_elt, param)]
-    assert mocked_stderr.call_args_list == [call(tr_elt, param)]
+    assert mocked_clear.call_args_list == [call(tr_elt, param, True)]
+    assert mocked_stdout.call_args_list == [call(tr_elt, param, True)]
+    assert mocked_stderr.call_args_list == [call(tr_elt, param, True)]
+    mocker.resetall()
+    tr_elt.reset_all()
+    handler.view_ctx.format_url.reset_mock()
+    # test call on selected process NOT having a stdout_logfile
+    param = {'namespec': 'dummy_application:dummy_process_2', 'identifier': '10.0.0.1',
+             'process_name': 'dummy_process_1'}
+    handler.write_common_process_status(tr_elt, param, True)
+    assert mocked_state.call_args_list == [call(tr_elt, param)]
+    assert mocked_stats.call_args_list == [call(tr_elt, param)]
+    assert name_td_elt.content.call_args_list == [call('\u21B3 dummy_process_1')]
+    assert not name_a_elt.content.called
+    assert not handler.view_ctx.format_url.called
+    assert not name_a_elt.attributes.called
+    assert mocked_start.call_args_list == [call(tr_elt, param)]
+    assert mocked_stop.call_args_list == [call(tr_elt, param)]
+    assert mocked_restart.call_args_list == [call(tr_elt, param)]
+    assert mocked_clear.call_args_list == [call(tr_elt, param, True)]
+    assert mocked_stdout.call_args_list == [call(tr_elt, param, True)]
+    assert mocked_stderr.call_args_list == [call(tr_elt, param, True)]
+    mocker.resetall()
+    tr_elt.reset_all()
+    handler.view_ctx.format_url.reset_mock()
+    # test call on selected process NOT having a stdout_logfile but without check required
+    param = {'namespec': 'dummy_application:dummy_process_2', 'identifier': '10.0.0.1',
+             'process_name': 'dummy_process_1'}
+    handler.write_common_process_status(tr_elt, param, False)
+    assert mocked_state.call_args_list == [call(tr_elt, param)]
+    assert mocked_stats.call_args_list == [call(tr_elt, param)]
+    assert not name_td_elt.content.called
+    assert name_a_elt.content.call_args_list == [call('\u21B3 dummy_process_1')]
+    assert handler.view_ctx.format_url.call_args_list == [call('10.0.0.1', 'tail.html',
+                                                               processname='dummy_application:dummy_process_2',
+                                                               limit=1024)]
+    assert name_a_elt.attributes.call_args_list == [call(href='an url', target="_blank")]
+    assert mocked_start.call_args_list == [call(tr_elt, param)]
+    assert mocked_stop.call_args_list == [call(tr_elt, param)]
+    assert mocked_restart.call_args_list == [call(tr_elt, param)]
+    assert mocked_clear.call_args_list == [call(tr_elt, param, False)]
+    assert mocked_stdout.call_args_list == [call(tr_elt, param, False)]
+    assert mocked_stderr.call_args_list == [call(tr_elt, param, False)]
 
 
 def test_write_detailed_process_cpu(handler):
@@ -983,6 +1047,32 @@ def test_make_callback(handler):
         handler.make_callback('dummy_namespec', 'dummy_action')
 
 
+def test_multicall_rpc_action(mocker, handler):
+    """ Test the multicall_rpc_action method. """
+    mocked_rpc = mocker.patch('supvisors.viewhandler.generic_rpc', return_value='a deferred result')
+    multicall = [{'methodName': 'supervisor.stopProcessGroup', 'params': ['dummy_proc']},
+                 {'methodName': 'supervisor.startProcessGroup', 'params': ['dummy_proc', False]}]
+    assert handler.multicall_rpc_action(multicall, 'successful') == 'a deferred result'
+    assert mocked_rpc.call_args_list == [call(handler.supvisors.supervisor_data.system_rpc_interface,
+                                              'multicall', (multicall,), 'successful')]
+
+
+def test_supervisor_rpc_action(mocker, handler):
+    """ Test the supervisor_rpc_action method. """
+    mocked_rpc = mocker.patch('supvisors.viewhandler.generic_rpc', return_value='a deferred result')
+    assert handler.supervisor_rpc_action('startProcess', ('dummy_proc', True), 'successful') == 'a deferred result'
+    assert mocked_rpc.call_args_list == [call(handler.supvisors.supervisor_data.supervisor_rpc_interface,
+                                              'startProcess', ('dummy_proc', True), 'successful')]
+
+
+def test_supvisors_rpc_action(mocker, handler):
+    """ Test the supvisors_rpc_action method. """
+    mocked_rpc = mocker.patch('supvisors.viewhandler.generic_rpc', return_value='a deferred result')
+    assert handler.supvisors_rpc_action('start_process', (1, 'dummy_proc', True), 'successful') == 'a deferred result'
+    assert mocked_rpc.call_args_list == [call(handler.supvisors.supervisor_data.supvisors_rpc_interface,
+                                              'start_process', (1, 'dummy_proc', True), 'successful')]
+
+
 def test_set_slope_class():
     """ Test the set_slope_class method. """
     elt = Mock(attrib={})
