@@ -17,15 +17,13 @@
 # limitations under the License.
 # ======================================================================
 
-import pytest
-
-from supervisor.web import MeldView
-
 from unittest.mock import call, patch, Mock
 
+import pytest
+from supervisor.web import MeldView
+
 from supvisors.viewsupvisors import *
 from supvisors.webutils import PROC_INSTANCE_PAGE, SUPVISORS_PAGE
-
 from .base import DummyHttpContext
 from .conftest import create_element
 
@@ -46,6 +44,7 @@ def view(http_context):
     view.view_ctx = Mock(parameters={}, **{'format_url.return_value': 'an url'})
     return view
 
+
 def test_init(view):
     """ Test the values set at construction. """
     # test instance inheritance
@@ -70,20 +69,48 @@ def test_write_navigation(mocker, view):
     assert mocked_nav.call_args_list == [call(mocked_root)]
 
 
-def test_write_header(view):
+def test_write_header(mocker, view):
     """ Test the write_header method. """
     # patch context
-    view.supvisors.fsm.state = SupvisorsStates.OPERATION
+    mocker.patch.object(view.sup_ctx, 'get_state_modes',
+                        return_value={'fsm_statename': SupvisorsStates.DEPLOYMENT.name,
+                                      'starting_jobs': True, 'stopping_jobs': False})
+    mocked_actions = mocker.patch.object(view, 'write_supvisors_actions')
     # build root structure
-    mocked_state_mid = Mock()
-    mocked_root = Mock(**{'findmeld.return_value': mocked_state_mid})
+    state_mid = create_element()
+    starting_mid = create_element()
+    stopping_mid = create_element()
+    mocked_root = create_element({'state_mid': state_mid, 'starting_mid': starting_mid, 'stopping_mid': stopping_mid})
     # test call with no auto-refresh
     view.write_header(mocked_root)
-    assert mocked_state_mid.content.call_args_list == [call('OPERATION')]
+    assert mocked_root.findmeld.call_args_list == [call('state_mid'), call('starting_mid'), call('stopping_mid')]
+    assert state_mid.content.call_args_list == [call('DEPLOYMENT')]
+    assert starting_mid.attrib['class'] == 'blink'
+    assert not starting_mid.replace.called
+    assert stopping_mid.attrib['class'] == ''
+    assert stopping_mid.replace.call_args_list == [call('')]
+    assert mocked_actions.call_args_list == [call(mocked_root)]
+
+
+def test_write_supvisors_actions(view):
+    """ Test the SupvisorsView.write_supvisors_actions method. """
+    # set context (meant to be set through render)
+    view.view_ctx = Mock(**{'format_url.return_value': 'an url'})
+    # build root structure
+    restart_mid = create_element()
+    shutdown_mid = create_element()
+    mocked_root = create_element({'restart_a_mid': restart_mid, 'shutdown_a_mid': shutdown_mid})
+    # test call
+    view.write_supvisors_actions(mocked_root)
+    assert mocked_root.findmeld.call_args_list == [call('restart_a_mid'), call('shutdown_a_mid')]
+    assert view.view_ctx.format_url.call_args_list == [call('', SUPVISORS_PAGE, **{ACTION: 'sup_restart'}),
+                                                       call('', SUPVISORS_PAGE, **{ACTION: 'sup_shutdown'})]
+    assert restart_mid.attributes.call_args_list == [call(href='an url')]
+    assert shutdown_mid.attributes.call_args_list == [call(href='an url')]
 
 
 def test_write_contents(mocker, view):
-    """ Test the write_contents method. """
+    """ Test the SupvisorsView.write_contents method. """
     mocked_boxes = mocker.patch('supvisors.viewsupvisors.SupvisorsView.write_instance_boxes')
     mocked_conflicts = mocker.patch('supvisors.viewsupvisors.SupvisorsView.write_conciliation_table')
     mocked_strategies = mocker.patch('supvisors.viewsupvisors.SupvisorsView.write_conciliation_strategies')
@@ -557,7 +584,7 @@ def test_stop_action(mocker, view):
     process = Mock(running_identifiers=['10.0.0.1', '10.0.0.2', '10.0.0.3'])
     mocked_get = mocker.patch.object(view.supvisors.context, 'get_process', return_value=process)
     # test call
-    mocked_rpc = mocker.patch.object(view.supvisors.zmq.pusher, 'send_stop_process')
+    mocked_rpc = mocker.patch.object(view.supvisors.sockets.pusher, 'send_stop_process')
     cb = view.stop_action('dummy_proc', '10.0.0.2')
     assert callable(cb)
     assert mocked_rpc.call_args_list == [call('10.0.0.2', 'dummy_proc')]
@@ -577,7 +604,7 @@ def test_keep_action(mocker, view):
     process = Mock(running_identifiers=['10.0.0.1', '10.0.0.2', '10.0.0.3'])
     mocked_get = mocker.patch.object(view.supvisors.context, 'get_process', return_value=process)
     # test call
-    with patch.object(view.supvisors.zmq.pusher, 'send_stop_process') as mocked_rpc:
+    with patch.object(view.supvisors.sockets.pusher, 'send_stop_process') as mocked_rpc:
         cb = view.keep_action('dummy_proc', '10.0.0.2')
     assert callable(cb)
     assert mocked_rpc.call_args_list == [call('10.0.0.1', 'dummy_proc'), call('10.0.0.3', 'dummy_proc')]
