@@ -18,14 +18,14 @@
 # ======================================================================
 
 import time
+from typing import Type
 
 from supervisor.compat import as_bytes, as_string
-from supervisor.http import NOT_DONE_YET
 from supervisor.states import SupervisorStates, RUNNING_STATES, STOPPED_STATES
 from supervisor.web import MeldView
 
 from .rpcinterface import API_VERSION
-from .ttypes import SupvisorsInstanceStates, SupvisorsStates, Payload, ProcessHistoryStats
+from .ttypes import SupvisorsInstanceStates, SupvisorsStates, Payload, PayloadList, ProcessHistoryStats
 from .utils import get_stats
 from .viewcontext import *
 from .viewimage import process_cpu_img, process_mem_img
@@ -47,6 +47,8 @@ class ViewHandler(MeldView):
         self.sup_ctx = self.supvisors.context
         # keep reference to the local node name
         self.local_identifier = self.supvisors.supvisors_mapper.local_identifier
+        # define statistics visibility
+        self.has_statistics = True
         # init view_ctx (only for tests)
         self.view_ctx = None
 
@@ -146,6 +148,8 @@ class ViewHandler(MeldView):
                     update_attrib(li_elt, 'class', 'active')
                 # set hyperlink attributes
                 elt = li_elt.findmeld('instance_a_mid')
+                if status.state_modes.starting_jobs or status.state_modes.stopping_jobs:
+                    update_attrib(elt, 'class', 'blink')
                 if status.state == SupvisorsInstanceStates.RUNNING:
                     # go to web page located on the Supvisors instance so as to reuse Supervisor StatusView
                     url = self.view_ctx.format_url(item, PROC_INSTANCE_PAGE)
@@ -165,6 +169,8 @@ class ViewHandler(MeldView):
         # write applications
         mid_elt = root.findmeld('appli_li_mid')
         applications = self.sup_ctx.get_managed_applications().values()
+        working_apps = (self.supvisors.starter.get_application_job_names()
+                        | self.supvisors.stopper.get_application_job_names())
         # forced to list otherwise not easily testable
         for li_elt, item in mid_elt.repeat(sorted(applications, key=lambda x: x.application_name)):
             failure = item.major_failure or item.minor_failure
@@ -177,7 +183,9 @@ class ViewHandler(MeldView):
                 update_attrib(li_elt, 'class', 'failure')
             # set hyperlink attributes
             elt = li_elt.findmeld('appli_a_mid')
-            if self.supvisors.fsm.state == SupvisorsStates.INITIALIZATION:
+            if item.application_name in working_apps:
+                update_attrib(elt, 'class', 'blink')
+            if self.supvisors.fsm.state in [SupvisorsStates.OFF, SupvisorsStates.INITIALIZATION]:
                 update_attrib(elt, 'class', 'off')
             else:
                 # force default application starting strategy
@@ -197,7 +205,7 @@ class ViewHandler(MeldView):
 
     def write_periods(self, root):
         """ Write configured periods for statistics. """
-        if self.supvisors.options.stats_enabled:
+        if self.has_statistics:
             # write the available periods
             mid_elt = root.findmeld('period_li_mid')
             for li_elt, item in mid_elt.repeat(self.supvisors.options.stats_periods):
@@ -221,7 +229,7 @@ class ViewHandler(MeldView):
     def write_common_process_cpu(self, tr_elt, info):
         """ Write the CPU part of the common process status.
         Statistics data comes from node. """
-        if self.supvisors.options.stats_enabled:
+        if self.has_statistics:
             proc_stats = info['proc_stats']
             elt = tr_elt.findmeld('pcpu_a_mid')
             if proc_stats and len(proc_stats[0]) > 0:
@@ -251,7 +259,7 @@ class ViewHandler(MeldView):
     def write_common_process_mem(self, tr_elt, info: Payload) -> None:
         """ Write the MEM part of the common process status.
         Statistics data comes from node. """
-        if self.supvisors.options.stats_enabled:
+        if self.has_statistics:
             proc_stats = info['proc_stats']
             elt = tr_elt.findmeld('pmem_a_mid')
             if proc_stats and len(proc_stats[1]) > 0:
@@ -278,77 +286,85 @@ class ViewHandler(MeldView):
 
     def write_process_start_button(self, tr_elt, info):
         """ Write the configuration of the start button of a process.
-        The action will be handled by the local supvisors. """
-        self._write_process_button(tr_elt, 'start_a_mid', '', self.page_name,
-                                   'start', info['namespec'], info['statecode'], STOPPED_STATES)
+        The action will be handled by the local Supvisors instance. """
+        self._write_process_button(tr_elt, 'start_a_mid', '', self.page_name, 'start', info['namespec'],
+                                   info['startable'] and info['statecode'] in STOPPED_STATES)
 
     def write_process_stop_button(self, tr_elt, info):
         """ Write the configuration of the stop button of a process.
         The action will be handled by the local supvisors. """
-        self._write_process_button(tr_elt, 'stop_a_mid', '', self.page_name,
-                                   'stop', info['namespec'], info['statecode'], RUNNING_STATES)
+        self._write_process_button(tr_elt, 'stop_a_mid', '', self.page_name, 'stop', info['namespec'],
+                                   info['statecode'] in RUNNING_STATES)
 
     def write_process_restart_button(self, tr_elt, info):
         """ Write the configuration of the restart button of a process.
         The action will be handled by the local supvisors. """
-        self._write_process_button(tr_elt, 'restart_a_mid', '', self.page_name,
-                                   'restart', info['namespec'], info['statecode'], RUNNING_STATES)
+        self._write_process_button(tr_elt, 'restart_a_mid', '', self.page_name, 'restart', info['namespec'],
+                                   info['startable'] and info['statecode'] in RUNNING_STATES)
 
-    def write_process_clear_button(self, tr_elt, info):
+    def write_process_clear_button(self, tr_elt, info, check_logfiles: bool):
         """ Write the configuration of the clear logs button of a process.
         This action must be sent to the relevant node. """
+        namespec = info['namespec']
+        has_stdout = not check_logfiles or self.supvisors.supervisor_data.has_logfile(namespec, 'stdout')
+        has_stderr = not check_logfiles or self.supvisors.supervisor_data.has_logfile(namespec, 'stderr')
         self._write_process_button(tr_elt, 'clear_a_mid', info['identifier'], self.page_name,
-                                   'clearlog', info['namespec'], '', '')
+                                   'clearlog', namespec, has_stdout or has_stderr)
 
-    def write_process_stdout_button(self, tr_elt, info):
+    def write_process_stdout_button(self, tr_elt, info, check_logfiles: bool):
         """ Write the configuration of the tail stdout button of a process.
         This action must be sent to the relevant node. """
+        namespec = info['namespec']
+        has_stdout = not check_logfiles or self.supvisors.supervisor_data.has_logfile(namespec, 'stdout')
         # no action requested. page name is enough
         self._write_process_button(tr_elt, 'tailout_a_mid', info['identifier'],
-                                   STDOUT_PAGE % quote(info['namespec'] or ''),
-                                   '', info['namespec'], '', '')
+                                   STDOUT_PAGE % quote(namespec or ''), '', namespec, has_stdout)
 
-    def write_process_stderr_button(self, tr_elt, info):
+    def write_process_stderr_button(self, tr_elt, info, check_logfiles: bool):
         """ Write the configuration of the tail stderr button of a process.
         This action must be sent to the relevant node. """
+        namespec = info['namespec']
+        has_stderr = not check_logfiles or self.supvisors.supervisor_data.has_logfile(namespec, 'stderr')
         # no action requested. page name is enough
         self._write_process_button(tr_elt, 'tailerr_a_mid', info['identifier'],
-                                   STDERR_PAGE % quote(info['namespec'] or ''),
-                                   '', info['namespec'], '', '')
+                                   STDERR_PAGE % quote(namespec or ''), '', namespec, has_stderr)
 
-    def _write_process_button(self, tr_elt, elt_name, identifier, page, action, namespec, state, state_list):
+    def _write_process_button(self, tr_elt, elt_name: str, identifier: str, page: str, action: str, namespec: str,
+                              active: bool = True):
         """ Write the configuration of a process button. """
         elt = tr_elt.findmeld(elt_name)
-        if namespec:
-            if state in state_list:
-                update_attrib(elt, 'class', 'button on')
-                url = self.view_ctx.format_url(identifier, page, **{ACTION: action, NAMESPEC: namespec})
-                elt.attributes(href=url)
-            else:
-                update_attrib(elt, 'class', 'button off')
+        if active:
+            update_attrib(elt, 'class', 'button on')
+            url = self.view_ctx.format_url(identifier, page, **{ACTION: action, NAMESPEC: namespec})
+            elt.attributes(href=url)
         else:
-            # this corresponds to an application row: no action available
-            elt.content('')
+            update_attrib(elt, 'class', 'button off')
 
     def write_common_process_table(self, table_elt):
         """ Hide MEM+CPU head+foot cells if statistics disabled"""
-        if not self.supvisors.options.stats_enabled:
+        if not self.has_statistics:
             for mid in ['mem_head_th_mid', 'cpu_head_th_mid', 'mem_foot_th_mid', 'cpu_foot_th_mid', 'total_mid']:
                 elt = table_elt.findmeld(mid)
                 if elt is not None:
                     elt.deparent()
 
-    def write_common_status(self, tr_elt, info: Payload) -> None:
-        """ Write the common part of a process or application status into a table. """
+    @staticmethod
+    def write_common_state(tr_elt, info: Payload) -> None:
+        """ Write the common part of a process state into a table. """
         # print state
         elt = tr_elt.findmeld('state_td_mid')
         update_attrib(elt, 'class', info['gravity'])
         if info['has_crashed']:
             update_attrib(elt, 'class', 'crashed')
+        if info['disabled']:
+            update_attrib(elt, 'class', 'disabled')
         elt.content(info['statename'])
         # print description
         elt = tr_elt.findmeld('desc_td_mid')
         elt.content(info['description'])
+
+    def write_common_statistics(self, tr_elt, info: Payload) -> None:
+        """ Write the common part of a process or application statistics into a table. """
         # print expected load
         elt = tr_elt.findmeld('load_td_mid')
         elt.content(f"{info['expected_load']}%")
@@ -356,27 +372,37 @@ class ViewHandler(MeldView):
         self.write_common_process_cpu(tr_elt, info)
         self.write_common_process_mem(tr_elt, info)
 
-    def write_common_process_status(self, tr_elt, info: Payload) -> None:
+    def write_common_process_status(self, tr_elt, info: Payload, check_logfiles: bool = True) -> None:
         """ Write the common part of a process status into a table. """
         # print common status
-        self.write_common_status(tr_elt, info)
+        self.write_common_state(tr_elt, info)
+        self.write_common_statistics(tr_elt, info)
         # print process name
         # add break character only to processes belonging to an application
         process_name = info['process_name']
         if 'single' not in info or not info['single']:
             process_name = '\u21B3 ' + process_name
-        elt = tr_elt.findmeld('name_a_mid')
-        elt.content(process_name)
-        url = self.view_ctx.format_url(info['identifier'], TAIL_PAGE, **{PROCESS: info['namespec']})
-        elt.attributes(href=url, target="_blank")
+        name_elt = tr_elt.findmeld('name_td_mid')
+        # tail hyperlink depends on logfile availability
+        namespec = info['namespec']
+        # for application page, cannot state if logfile is there
+        if not check_logfiles or self.supvisors.supervisor_data.has_logfile(namespec, 'stdout'):
+            elt = name_elt.findmeld('name_a_mid')
+            elt.content(process_name)
+            url = self.view_ctx.format_url(info['identifier'], TAIL_PAGE,
+                                           **{PROCESS: namespec, LIMIT: self.supvisors.options.tail_limit})
+            elt.attributes(href=url, target="_blank")
+        else:
+            # overwrite a href entry
+            name_elt.content(process_name)
         # manage actions iaw state
         self.write_process_start_button(tr_elt, info)
         self.write_process_stop_button(tr_elt, info)
         self.write_process_restart_button(tr_elt, info)
         # manage log actions
-        self.write_process_clear_button(tr_elt, info)
-        self.write_process_stdout_button(tr_elt, info)
-        self.write_process_stderr_button(tr_elt, info)
+        self.write_process_clear_button(tr_elt, info, check_logfiles)
+        self.write_process_stdout_button(tr_elt, info, check_logfiles)
+        self.write_process_stderr_button(tr_elt, info, check_logfiles)
 
     def write_detailed_process_cpu(self, stats_elt, proc_stats: ProcessHistoryStats, nb_cores: int) -> bool:
         """ Write the CPU part of the detailed process status.
@@ -423,18 +449,18 @@ class ViewHandler(MeldView):
             elt = stats_elt.findmeld('pmemval_td_mid')
             if rate is not None:
                 self.set_slope_class(elt, rate)
-            elt.content('{:.2f}%'.format(proc_stats[1][-1]))
+            elt.content(f'{proc_stats[1][-1]:.2f}%')
             # set mean value
             elt = stats_elt.findmeld('pmemavg_td_mid')
-            elt.content('{:.2f}%'.format(avg))
+            elt.content(f'{avg:.2f}%')
             if a is not None:
                 # set slope value between last 2 values
                 elt = stats_elt.findmeld('pmemslope_td_mid')
-                elt.content('{:.2f}'.format(a))
+                elt.content(f'{a:.2f}')
             if dev is not None:
                 # set standard deviation
                 elt = stats_elt.findmeld('pmemdev_td_mid')
-                elt.content('{:.2f}'.format(dev))
+                elt.content(f'{dev:.2f}')
             return True
 
     def write_process_plots(self, proc_stats: ProcessHistoryStats, nb_cores: int) -> None:
@@ -442,14 +468,14 @@ class ViewHandler(MeldView):
         try:
             from supvisors.plot import StatisticsPlot
             # build CPU image
-            cpu_img = StatisticsPlot()
+            cpu_img = StatisticsPlot(self.logger)
             cpu_values = proc_stats[0]
             if not self.supvisors.options.stats_irix_mode:
                 cpu_values = [x / nb_cores for x in cpu_values]
             cpu_img.add_plot('CPU', '%', cpu_values)
             cpu_img.export_image(process_cpu_img)
             # build Memory image
-            mem_img = StatisticsPlot()
+            mem_img = StatisticsPlot(self.logger)
             mem_img.add_plot('MEM', '%', proc_stats[1])
             mem_img.export_image(process_mem_img)
         except ImportError:
@@ -480,8 +506,11 @@ class ViewHandler(MeldView):
             # remove stats part if empty
             stats_elt.replace('')
 
-    def handle_action(self):
-        """ Handling of the actions requested from the Supvisors web pages. """
+    def handle_action(self) -> Optional[Type[NOT_DONE_YET]]:
+        """ Handling of the actions requested from the Supvisors web pages.
+
+        :return: NOT_DONE_YET if action is deferred, None otherwise
+        """
         # check if any action is requested
         action = self.view_ctx.get_action()
         if action:
@@ -497,13 +526,50 @@ class ViewHandler(MeldView):
             if message is not None:
                 self.view_ctx.store_message = format_gravity_message(message)
 
-    def make_callback(self, namespec: str, action: str):
+    def make_callback(self, namespec: str, action: str) -> Callable:
         """ Triggers processing iaw action requested.
-        Subclasses will define what's to be done. """
+        Subclasses will define what's to be done.
+
+        :param namespec: the optional process namespec to which the action applies
+        :param action: the action to perform
+        :return: a callable for deferred result
+        """
         raise NotImplementedError
 
+    def multicall_rpc_action(self, args: PayloadList, success_msg: str) -> Callable:
+        """ Generic wrapper for a System Supervisor RPC.
+
+        :param args: the multiple RPC parameters
+        :param success_msg: the message in case of success
+        :return: a callable for deferred result
+        """
+        rpc_intf = self.supvisors.supervisor_data.system_rpc_interface
+        return generic_rpc(rpc_intf, 'multicall', (args,), success_msg)
+
+    def supervisor_rpc_action(self, rpc_name: str, args: tuple, success_msg: str) -> Callable:
+        """ Generic wrapper for a Supervisor RPC.
+
+        :param rpc_name: the RPC name in the Supervisor XML-RPC API
+        :param args: the arguments of the RPC
+        :param success_msg: the message in case of success
+        :return: a callable for deferred result
+        """
+        rpc_intf = self.supvisors.supervisor_data.supervisor_rpc_interface
+        return generic_rpc(rpc_intf, rpc_name, args, success_msg)
+
+    def supvisors_rpc_action(self, rpc_name: str, args: tuple, success_msg: str) -> Callable:
+        """ Generic wrapper for a Supvisors RPC.
+
+        :param rpc_name: the RPC name in the Supvisors XML-RPC API
+        :param args: the arguments of the RPC
+        :param success_msg: the message in case of success
+        :return: a callable for deferred result
+        """
+        rpc_intf = self.supvisors.supervisor_data.supvisors_rpc_interface
+        return generic_rpc(rpc_intf, rpc_name, args, success_msg)
+
     @staticmethod
-    def set_slope_class(elt, value):
+    def set_slope_class(elt, value: float) -> None:
         """ Set attribute class iaw positive or negative slope. """
         if abs(value) < .005:
             update_attrib(elt, 'class', 'stable')
