@@ -17,14 +17,15 @@
 # limitations under the License.
 # ======================================================================
 
-from supervisor.states import RUNNING_STATES
-from supervisor.xmlrpc import RPCError
+import os
+
+from supervisor.states import ProcessStates, RUNNING_STATES
 
 from .application import ApplicationStatus
 from .instancestatus import SupvisorsInstanceStatus
-from .ttypes import Payload, PayloadList, ProcessHistoryStats
+from .ttypes import SupvisorsFaults, Payload, PayloadList, ProcessHistoryStats
 from .viewcontext import *
-from .viewsupstatus import SupvisorsInstanceView
+from .viewinstance import SupvisorsInstanceView
 from .webutils import *
 
 
@@ -78,13 +79,34 @@ class ProcInstanceView(SupvisorsInstanceView):
             nb_cores, proc_stats = self.view_ctx.get_process_stats(namespec)
             payload = {'application_name': info['group'], 'process_name': info['name'], 'namespec': namespec,
                        'single': single, 'identifier': self.view_ctx.local_identifier,
+                       'disabled': info['disabled'], 'startable': not info['disabled'],
                        'statename': info['statename'], 'statecode': info['state'],
                        'gravity': 'FATAL' if crashed else info['statename'], 'has_crashed': info['has_crashed'],
                        'description': info['description'], 'expected_load': expected_load,
                        'nb_cores': nb_cores, 'proc_stats': proc_stats}
             data.append(payload)
         # re-arrange data
-        return self.sort_data(data)
+        sorted_data, excluded_data = self.sort_data(data)
+        # add supervisord payload at the end of sorted data
+        sorted_data.append(self.get_supervisord_data(status))
+        return sorted_data, excluded_data
+
+    def get_supervisord_data(self, status: SupvisorsInstanceStatus) -> Payload:
+        """ Collect sorted data on supervisord process.
+
+        :param status: the local Supvisors instance
+        :return: the supervisord data.
+        """
+        nb_cores, proc_stats = self.view_ctx.get_process_stats('supervisord')
+        payload = {'application_name': 'supervisord', 'process_name': 'supervisord', 'namespec': 'supervisord',
+                   'single': True, 'identifier': self.view_ctx.local_identifier, 'disabled': False, 'startable': False,
+                   'statename': 'RUNNING', 'statecode': 20, 'gravity': 'RUNNING', 'has_crashed': False,
+                   'expected_load': 0, 'nb_cores': nb_cores, 'proc_stats': proc_stats}
+        # add description (pid / uptime) as done by Supervisor
+        info = {'state': ProcessStates.RUNNING, 'start': status.start_time, 'now': status.local_time,
+                'pid': os.getpid()}
+        payload['description'] = ProcessStatus.update_description(info)
+        return payload
 
     def sort_data(self, data: PayloadList) -> Tuple[PayloadList, PayloadList]:
         """ This method sorts a process list by application and using the alphabetical order.
@@ -114,14 +136,6 @@ class ProcInstanceView(SupvisorsInstanceView):
             else:
                 # push to excluded data
                 excluded_data.extend(application_processes)
-        # add supervisord payload at the end
-        nb_cores, proc_stats = self.view_ctx.get_process_stats('supervisord')
-        payload = {'application_name': 'supervisord', 'process_name': 'supervisord', 'namespec': 'supervisord',
-                   'single': True, 'identifier': self.view_ctx.local_identifier,
-                   'statename': 'RUNNING', 'statecode': 20, 'gravity': 'RUNNING', 'has_crashed': False,
-                   'description': f'Supervisor {self.view_ctx.local_identifier}', 'expected_load': 0,
-                   'nb_cores': nb_cores, 'proc_stats': proc_stats}
-        sorted_data.append(payload)
         return sorted_data, excluded_data
 
     def get_application_summary(self, application_name: str, application_processes: PayloadList) -> Payload:
@@ -135,7 +149,7 @@ class ProcInstanceView(SupvisorsInstanceView):
         # create application payload
         application = self.sup_ctx.applications[application_name]
         payload = {'application_name': application_name, 'process_name': None, 'namespec': None,
-                   'identifier': self.view_ctx.local_identifier,
+                   'identifier': self.view_ctx.local_identifier, 'disabled': False, 'startable': False,
                    'statename': application.state.name, 'statecode': application.state.value,
                    'gravity': application.state.name, 'has_crashed': False,
                    'description': application.get_operational_status(),
@@ -247,8 +261,6 @@ class ProcInstanceView(SupvisorsInstanceView):
     def write_application_status(self, tr_elt, info, shaded_tr):
         """ Write the application section into a table. """
         application_name = info['application_name']
-        # print common status
-        self.write_common_status(tr_elt, info)
         # print application shex
         elt = tr_elt.findmeld('shex_td_mid')
         application_shex, inverted_shex = self.view_ctx.get_application_shex(application_name)
@@ -261,17 +273,26 @@ class ProcInstanceView(SupvisorsInstanceView):
         elt.content(f'{SHEX_SHRINK if application_shex else SHEX_EXPAND}')
         url = self.view_ctx.format_url('', self.page_name, **{SHRINK_EXPAND: inverted_shex})
         elt.attributes(href=url)
-        # print application name
-        elt = tr_elt.findmeld('name_a_mid')
+        # print application name (covers state and description)
+        elt = tr_elt.findmeld('name_td_mid')
+        elt.attrib['colspan'] = '3'
+        elt = elt.findmeld('name_a_mid')
         elt.content(application_name)
         url = self.view_ctx.format_url('', APPLICATION_PAGE, **{APPLI: application_name})
         elt.attributes(href=url)
-        # remove all actions
-        for mid in ['start_td_mid', 'clear_td_mid']:
-            elt = tr_elt.findmeld(mid)
-            elt.attrib['colspan'] = '3'
-            elt.content('')
-        for mid in ['stop_td_mid', 'restart_td_mid', 'tailout_td_mid', 'tailerr_td_mid']:
+        for mid in ['state_td_mid', 'desc_td_mid']:
+            tr_elt.findmeld(mid).replace('')
+        # print application statistics
+        self.write_common_statistics(tr_elt, info)
+        # start / stop / restart group actions
+        self._write_process_button(tr_elt, 'start_a_mid', '', self.page_name, 'startgroup', f'{application_name}:')
+        self._write_process_button(tr_elt, 'stop_a_mid', '', self.page_name, 'stopgroup', f'{application_name}:')
+        self._write_process_button(tr_elt, 'restart_a_mid', '', self.page_name, 'restartgroup', f'{application_name}:')
+        # remove log actions
+        elt = tr_elt.findmeld('clear_td_mid')
+        elt.attrib['colspan'] = '3'
+        elt.content('')
+        for mid in ['tailout_td_mid', 'tailerr_td_mid']:
             tr_elt.findmeld(mid).replace('')
 
     def write_supervisord_status(self, tr_elt, info: Payload) -> None:
@@ -280,30 +301,38 @@ class ProcInstanceView(SupvisorsInstanceView):
         if self.sup_ctx.is_master:
             elt = tr_elt.findmeld('shex_td_mid')
             elt.content(MASTER_SYMBOL)
-        # print common status
-        self.write_common_status(tr_elt, info)
         # print process name
         elt = tr_elt.findmeld('name_a_mid')
         elt.content(info['process_name'])
-        url = self.view_ctx.format_url('', MAIN_TAIL_PAGE, **{PROCESS: info['namespec']})
+        url = self.view_ctx.format_url('', MAIN_TAIL_PAGE, **{PROCESS: info['namespec'],
+                                                              LIMIT: self.supvisors.options.tail_limit})
         elt.attributes(href=url, target="_blank")
+        # print common status
+        self.write_common_state(tr_elt, info)
+        self.write_common_statistics(tr_elt, info)
         # manage actions
-        self.write_supervisord_button(tr_elt, 'stop_a_mid', self.page_name, **{ACTION: 'shutdownsup'})
-        self.write_supervisord_button(tr_elt, 'restart_a_mid', self.page_name, **{ACTION: 'restartsup'})
+        self._write_supervisord_button(tr_elt, 'stop_a_mid', self.page_name, **{ACTION: 'shutdownsup'})
+        self._write_supervisord_button(tr_elt, 'restart_a_mid', self.page_name, **{ACTION: 'restartsup'})
         # manage log actions
-        self.write_supervisord_button(tr_elt, 'clear_a_mid', self.page_name, **{ACTION: 'mainclearlog'})
-        self.write_supervisord_button(tr_elt, 'tailout_a_mid', MAIN_STDOUT_PAGE)
+        self._write_supervisord_button(tr_elt, 'clear_a_mid', self.page_name, **{ACTION: 'mainclearlog'})
+        self._write_supervisord_button(tr_elt, 'tailout_a_mid', MAIN_STDOUT_PAGE)
         # start and tail stderr are not applicable
-        tr_elt.findmeld('start_a_mid').content('')
-        tr_elt.findmeld('tailerr_a_mid').content('')
+        self._write_supervisord_off_button(tr_elt, 'start_a_mid')
+        self._write_supervisord_off_button(tr_elt, 'tailerr_a_mid')
 
-    def write_supervisord_button(self, tr_elt, mid: str, page: str, **action) -> None:
+    def _write_supervisord_button(self, tr_elt, mid: str, page: str, **action) -> None:
         """ Write the configuration of the button of supervisord. """
         elt = tr_elt.findmeld(mid)
         update_attrib(elt, 'class', 'button on')
         url = self.view_ctx.format_url('', page, **action)
         elt.attributes(href=url)
 
+    @staticmethod
+    def _write_supervisord_off_button(tr_elt, mid: str) -> None:
+        """ Write the configuration of the button of supervisord. """
+        elt = tr_elt.findmeld(mid)
+        update_attrib(elt, 'class', 'button off')
+
     def write_total_status(self, table_elt, sorted_data: PayloadList, excluded_data: PayloadList):
         """ Write the total statistics for this Supvisors instance.
 
@@ -334,15 +363,63 @@ class ProcInstanceView(SupvisorsInstanceView):
     # ACTION part
     def make_callback(self, namespec, action):
         """ Triggers processing iaw action requested """
+        if action == 'startgroup':
+            return self.start_group_action(namespec)
+        if action == 'stopgroup':
+            return self.stop_group_action(namespec)
+        if action == 'restartgroup':
+            return self.restart_group_action(namespec)
         if action == 'mainclearlog':
             return self.clear_log_action()
-        return super().make_callback(namespec, action)
+        result = super().make_callback(namespec, action)
+        # WARN: this ugly part is necessary to handle the DISABLED exception that can be raised from Supervisor
+        #  startProcess. It is not possible to patch Supervisor make_callback without copying a huge piece of source
+        #  code from Supervisor, so here is an inspection of the make_callback result to check for a callable declaring
+        #  an 'unexpected rpc fault [103]'
+        if callable(result):
+            message = result()
+            if type(message) is str and f'[{SupvisorsFaults.DISABLED.value}]' in message:
+                return delayed_error(f'Process {namespec}: disabled')
+        return result
 
-    def clear_log_action(self):
-        """ Clear the log of Supervisor. """
-        rpc_intf = self.supvisors.supervisor_data.supervisor_rpc_interface
-        try:
-            rpc_intf.clearLog()
-        except RPCError as e:
-            return delayed_error(f'clearLog: {e.text}')
-        return delayed_info('Log for Supervisor cleared')
+    def start_group_action(self, namespec: str) -> Callable:
+        """ Start all processes in the group.
+        The RPC wait parameter is linked to the auto-refresh parameter of the page.
+
+        :param namespec: the group processes that have to be started (expecting a form like 'group:*')
+        :return: a callable for deferred result
+        """
+        self.logger.debug(f'ProcInstanceView.start_group_action: group_name={namespec}')
+        wait = not self.view_ctx.parameters[AUTO]
+        return self.supervisor_rpc_action('startProcess', (f'{namespec}', wait), f'Group {namespec} started')
+
+    def stop_group_action(self, namespec: str) -> Callable:
+        """ Stop all processes in the group.
+        The RPC wait parameter is linked to the auto-refresh parameter of the page.
+
+        :param namespec: the group processes that have to be stopped (expecting a form like 'group:*')
+        :return: a callable for deferred result
+        """
+        self.logger.debug(f'ProcInstanceView.stop_group_action: group_name={namespec}')
+        wait = not self.view_ctx.parameters[AUTO]
+        return self.supervisor_rpc_action('stopProcess', (f'{namespec}', wait), f'Group {namespec} stopped')
+
+    def restart_group_action(self, namespec: str) -> Callable:
+        """ Start all processes in the group.
+        The RPC wait parameter is linked to the auto-refresh parameter of the page and set only on the last call.
+
+        :param namespec: the group processes that have to be restarted (expecting a form like 'group:*')
+        :return: a callable for deferred result
+        """
+        self.logger.debug(f'ProcInstanceView.restart_group_action: group_name={namespec}')
+        wait = not self.view_ctx.parameters[AUTO]
+        multicall = [{'methodName': 'supervisor.stopProcess', 'params': [f'{namespec}']},
+                     {'methodName': 'supervisor.startProcess', 'params': [f'{namespec}', wait]}]
+        return self.multicall_rpc_action(multicall, f'Group {namespec} restarted')
+
+    def clear_log_action(self) -> Callable:
+        """ Clear the log of Supervisor.
+
+        :return: a callable for deferred result
+        """
+        return self.supervisor_rpc_action('clearLog', (), 'Log for Supervisor cleared')
