@@ -17,8 +17,6 @@
 # limitations under the License.
 # ======================================================================
 
-from supervisor.http import NOT_DONE_YET
-from supervisor.xmlrpc import RPCError
 from typing import Dict
 
 from .instancestatus import SupvisorsInstanceStatus
@@ -49,7 +47,7 @@ class SupvisorsView(ViewHandler):
         ViewHandler.__init__(self, context)
         self.page_name: str = SUPVISORS_PAGE
         # get applicable conciliation strategies
-        self.strategies = {str.lower(x) for x in enum_names(ConciliationStrategies)}
+        self.strategies = {x.name.lower() for x in ConciliationStrategies}
         self.strategies.remove(ConciliationStrategies.USER.name.lower())
         # global actions (no parameter)
         self.global_methods: SupvisorsView.ProcessCallableMap = {'sup_restart': self.sup_restart_action,
@@ -64,9 +62,30 @@ class SupvisorsView(ViewHandler):
 
     def write_header(self, root) -> None:
         """ Rendering of the header part of the Supvisors main page. """
+        state_modes = self.sup_ctx.get_state_modes()
         # set Supvisors state
         elt = root.findmeld('state_mid')
-        elt.content(self.supvisors.fsm.state.name)
+        elt.content(state_modes['fsm_statename'])
+        # set Supvisors modes
+        for mid, attr in [('starting_mid', 'starting_jobs'), ('stopping_mid', 'stopping_jobs')]:
+            elt = root.findmeld(mid)
+            if state_modes[attr]:
+                update_attrib(elt, 'class', 'blink')
+            else:
+                elt.replace('')
+        # write actions related to Supvisors
+        self.write_supvisors_actions(root)
+
+    def write_supvisors_actions(self, root) -> None:
+        """ Write actions related to Supvisors. """
+        # configure restart button
+        elt = root.findmeld('restart_a_mid')
+        url = self.view_ctx.format_url('', SUPVISORS_PAGE, **{ACTION: 'sup_restart'})
+        elt.attributes(href=url)
+        # configure shutdown button
+        elt = root.findmeld('shutdown_a_mid')
+        url = self.view_ctx.format_url('', SUPVISORS_PAGE, **{ACTION: 'sup_shutdown'})
+        elt.attributes(href=url)
 
     def write_contents(self, root) -> None:
         """ Rendering of the contents of the Supvisors main page.
@@ -316,7 +335,7 @@ class SupvisorsView(ViewHandler):
         """ Stop the conflicting process. """
         # get running instances of process
         running_identifiers = self.sup_ctx.get_process(namespec).running_identifiers
-        self.supvisors.zmq.pusher.send_stop_process(identifier, namespec)
+        self.supvisors.sockets.pusher.send_stop_process(identifier, namespec)
 
         def on_wait():
             if identifier in running_identifiers:
@@ -334,7 +353,7 @@ class SupvisorsView(ViewHandler):
         identifiers = running_identifiers.copy()
         identifiers.remove(kept_identifier)
         for identifier in identifiers:
-            self.supvisors.zmq.pusher.send_stop_process(identifier, namespec)
+            self.supvisors.sockets.pusher.send_stop_process(identifier, namespec)
 
         def on_wait():
             if len(running_identifiers) > 1:
